package at.ac.tuwien.dbai.ges.solver;

import java.io.FileNotFoundException;
import java.io.PrintWriter;

/**
 * Writes logs to a file, System.out or both.
 *
 * @author Lucas Kletzander
 */
public class LogPrinter {

    private PrintWriter writer;
    private final boolean consoleOut;

    /**
     * Initialize log printer.
     * @param filename Name of the output file, null to use console only.
     * @param consoleOut Whether to print to console.
     */
    public LogPrinter(String filename, boolean consoleOut) {

        try {

            if (filename != null) {
                writer = new PrintWriter(filename);
            }

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }

        this.consoleOut = consoleOut;

    }

    /**
     * Print a line to the chosen outputs.
     * @param line Line to print
     */
    public void println(String line) {

        if (consoleOut) {
            System.out.println(line);
        }

        if (writer != null) {
            writer.println(line);
        }

    }

    /**
     * Closes the file writer.
     */
    public void close() {

        if (writer != null) {
            writer.flush();
            writer.close();
            writer = null;
        }

    }

}
