package at.ac.tuwien.dbai.ges.solver;

import java.util.Collection;
import java.util.Random;

/**
 * Contains various methods to obtain (pseudo) randomness.
 *
 * @author Lucas Kletzander
 */
public class RandomProvider {

    private static Random random;

    private RandomProvider() {}

    /**
     * Initialize the random provider.
     */
    public static void init() {
        random = new Random();
    }

    /**
     * Initialize the random provider with a specific seed.
     * @param seed The seed
     */
    public static void init(long seed) {
        random = new Random(seed);
    }

    public static Random getRandom() {
        return random;
    }

    public static int getIntExcept(int bound, int except) {

        int result;
        do {
            result = random.nextInt(bound);
        } while (result == except);

        return result;

    }

    public static <T> T getRandomElement(Collection<T> collection) {
        return collection.stream().skip(random.nextInt(collection.size())).findFirst().orElseThrow(() -> new RuntimeException("Error selecting random element from collection"));
    }

    public static <T> T getRandomElementExcept(Collection<T> collection, T except) {

        if (collection.size() < 2) {
            throw new IllegalArgumentException("Collection must have at least 2 elements");
        }

        T element;
        do {
            element = getRandomElement(collection);
        } while (element.equals(except));

        return element;

    }

    public static int getInBounds(int lowerInclusive, int upperInclusive) {
        return random.nextInt(Math.max(upperInclusive - lowerInclusive + 1, 1)) + lowerInclusive;
    }

    public static int getInBounds(int lowerInclusive, int upperInclusive, int step) {
        return getInBounds((lowerInclusive + step - 1) / step, upperInclusive / step) * step;
    }

    public static int getInBoundsExcept(int lowerInclusive, int upperInclusive, int except) {
        return getIntExcept(Math.max(upperInclusive - lowerInclusive + 1, 1), except - lowerInclusive) + lowerInclusive;
    }

}
