package at.ac.tuwien.dbai.ges.solver;

import at.ac.tuwien.dbai.ges.schema.SchedulingHorizon;
import at.ac.tuwien.dbai.ges.schema.solution.Schedule;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.Initialization;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.HorizonConverter;
import at.ac.tuwien.dbai.ges.solver.converter.SolutionConverter;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;

import java.io.IOException;

/**
 * Checks whether a solution is valid (else returns NaN) and calculates the soft constraint value.
 *
 * @author Lucas Kletzander
 */
public class SolutionChecker {

    public static void main(String[] args) {

        System.out.println("Parsing file...");

        try {

            SchedulingHorizon horizon = MarshallingUtil.readFromFile(args[0], SchedulingHorizon.class);
            Schedule schedule = MarshallingUtil.readFromFile(args[1], Schedule.class);

            ConversionContext context = new ConversionContext();
            HorizonConverter converter = new HorizonConverter(context);
            Instance instance = converter.createAndConvert(horizon);

            SolutionConverter solutionConverter = new SolutionConverter(context);
            solutionConverter.convert(schedule, context.solution);

            Initialization initialization = new Initialization(instance);
            double value = initialization.initialize(context.solution);

            instance.getConstraintStream(context.solution).forEach(System.out::println);
            System.out.println("TOTAL VALUE: " + value);

        } catch (IOException e) {
            e.printStackTrace();
        }

    }

}
