package at.ac.tuwien.dbai.ges.solver.algorithm.move;

import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.*;

import java.util.HashMap;
import java.util.Map;

/**
 * Initializes the evaluation of a solution (possibly empty or given solution).
 *
 * @author Lucas Kletzander
 */
public class Initialization extends Move {

    public Initialization(Instance instance) {
        super(instance);
    }

    @Override
    public boolean prepare(Solution solution) {

        Map<Integer, Shift> replacements = new HashMap<>();

        for (EmployeeSchedule schedule : solution.schedule.values()) {

            processEmployeeConstraints(i -> i.addEmployee(schedule.id));
            replacements.clear();

            for (int i = 0; i < schedule.shifts.length; i++) {

                Shift shift = schedule.shifts[i];
                replacements.put(i, shift);

                if (shift != null) {

                    schedule.overlap.addShift(i, shift);

                    if (i >= instance.history) {

                        processShiftConstraints(shift.type, instance.employeeDefinition.getEmployee(schedule.id), i, j -> j.addShift(schedule.id, shift));

                        int day = i;
                        processBreakArrangementConstraints(schedule, i, j -> j.addShift(shift));
                        shift.breaks.values().forEach(j -> {
                            processBreakConstraints(j, k -> k.addBreak(j));
                            schedule.overlap.addOccupied(day, j.start, j.end);
                        });

                    }

                }

            }

            constraints.add(schedule.overlap);
            processShiftArrangementConstraints(instance.employeeDefinition.getEmployee(schedule.id), i -> i.initRow(schedule.id, schedule.shifts, replacements));

        }

        for (TaskInstance t : solution.taskInstances.values()) {

            for (String employee : t.employees) {
                processTaskConstraints(t.demandId, instance.employeeDefinition.getEmployee(employee).skills, i -> i.addTask(t));
                t.parts.forEach(i -> solution.schedule.get(employee).overlap.addOccupied(t.day, i.start, i.end));
            }

        }

        instance.getConstraintStream(solution).forEach(i -> {
            i.evaluate();
            constraints.add(i);
        });

        return true;

    }

    public double initialize(Solution solution) {

        prepare(solution);
        double value = evaluate();
        execute(solution);
        return value;

    }

}
