package at.ac.tuwien.dbai.ges.solver.algorithm.move.selection;

import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.demand.TaskDemand;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;
import at.ac.tuwien.dbai.ges.solver.solution.TaskInstance;

/**
 * A selection strategy that is biased towards areas with more conflicts (measured by higher values of certain constraints).
 *
 * @author Lucas Kletzander
 */
public class BiasedSelectionStrategy implements SelectionStrategy {

    private final Instance instance;
    private final RandomSelectionStrategy randomStrategy;

    public BiasedSelectionStrategy(Instance instance) {
        this.instance = instance;
        randomStrategy = new RandomSelectionStrategy(instance);
    }

    @Override
    public EmployeeSchedule selectEmployee(Solution solution) {
        return selectEmployee(solution, null);
    }

    @Override
    public EmployeeSchedule selectEmployeeExcept(Solution solution, EmployeeSchedule except) {
        return selectEmployee(solution, except);
    }

    private EmployeeSchedule selectEmployee(Solution solution, EmployeeSchedule except) {

        EmployeeSchedule schedule = randomStrategy.selectEmployeeExcept(solution, except);

        while (true) {
            EmployeeSchedule newSchedule = randomStrategy.selectEmployeeExcept(solution, except);
            if (newSchedule.overlap.value > schedule.overlap.value) {
                schedule = newSchedule;
            } else {
                break;
            }
        }

        return schedule;

    }

    @Override
    public int selectDay(Solution solution) {
        return selectDay(solution, -1);
    }

    @Override
    public int selectDayExcept(Solution solution, int except) {
        return selectDay(solution, except);
    }

    public int selectDay(Solution solution, int except) {

        int day = randomStrategy.selectDayExcept(solution, except);

        while (true) {
            int newDay = randomStrategy.selectDayExcept(solution, except);
            if (instance.taskDemands.demand.get(newDay).stream().mapToDouble(TaskDemand::getConstraintValue).sum() > instance.taskDemands.demand.get(day).stream().mapToDouble(TaskDemand::getConstraintValue).sum()) {
                day = newDay;
            } else {
                break;
            }
        }

        return day;

    }

    @Override
    public TaskInstance selectTask(Solution solution) {

        TaskInstance task = randomStrategy.selectTask(solution);

        while (true) {
            TaskInstance newTask = randomStrategy.selectTask(solution);
            if (instance.taskDemands.idMap.get(newTask.demandId).getConstraintValue() > instance.taskDemands.idMap.get(task.demandId).getConstraintValue()) {
                task = newTask;
            } else {
                break;
            }
        }

        return task;

    }

}
