package at.ac.tuwien.dbai.ges.solver.algorithm.move.selection;

import at.ac.tuwien.dbai.ges.solver.RandomProvider;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;
import at.ac.tuwien.dbai.ges.solver.solution.TaskInstance;

/**
 * A selection strategy that selects randomly.
 *
 * @author Lucas Kletzander
 */
public class RandomSelectionStrategy implements SelectionStrategy {

    private final Instance instance;

    public RandomSelectionStrategy(Instance instance) {
        this.instance = instance;
    }

    @Override
    public EmployeeSchedule selectEmployee(Solution solution) {
        return solution.scheduleList.get(RandomProvider.getRandom().nextInt(solution.scheduleList.size()));
    }

    @Override
    public EmployeeSchedule selectEmployeeExcept(Solution solution, EmployeeSchedule except) {

        EmployeeSchedule schedule;
        do {
            schedule = selectEmployee(solution);
        } while (schedule.equals(except));

        return schedule;

    }

    @Override
    public int selectDay(Solution solution) {
        return instance.history + RandomProvider.getRandom().nextInt(instance.periodLength);
    }

    @Override
    public int selectDayExcept(Solution solution, int except) {
        return RandomProvider.getInBoundsExcept(instance.history, instance.periodLength + instance.history - 1, except);
    }

    @Override
    public TaskInstance selectTask(Solution solution) {
        return solution.taskList.get(RandomProvider.getRandom().nextInt(solution.taskList.size()));
    }

}
