package at.ac.tuwien.dbai.ges.solver.algorithm.move.shift;

import at.ac.tuwien.dbai.ges.solver.algorithm.move.Move;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.selection.SelectionStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;

import java.util.HashMap;
import java.util.Map;

/**
 * This move chooses an employee and a day and adds a shift if none is currently scheduled, else removes the shift.
 *
 * @author Lucas Kletzander
 */
public class AddOrRemoveShift extends Move {

    private final SelectionStrategy strategy;
    private final ShiftGenerator generator;

    private EmployeeSchedule schedule;
    private int day;
    private Map<Integer, Shift> replacements = new HashMap<>();

    public AddOrRemoveShift(Instance instance, SelectionStrategy strategy) {

        super(instance);
        this.strategy = strategy;
        generator = new ShiftGenerator(instance);

    }

    @Override
    public boolean prepare(Solution solution) {

        EmployeeSchedule schedule = strategy.selectEmployee(solution);
        int day = strategy.selectDay(solution);
        return prepare(schedule, day);

    }

    public boolean prepare(EmployeeSchedule schedule, int day) {

        if (schedule.fixed[day]) {
            return false;
        }

        this.schedule = schedule;
        this.day = day;
        Shift shift = null;

        if (schedule.shifts[day] == null) {
            shift = generator.generate();
        }

        replacements.clear();
        replacements.put(day, shift);

        Shift change = shift == null ? schedule.shifts[day] : shift;
        processShiftConstraints(change.type, instance.employeeDefinition.getEmployee(schedule.id), day, shift == null ? i -> i.removeShift(schedule.id, change) : i -> i.addShift(schedule.id, change));
        processShiftArrangementConstraints(instance.employeeDefinition.getEmployee(schedule.id), i -> i.changeRow(schedule.id, schedule.shifts, replacements));

        if (shift == null) {
            schedule.overlap.removeShift(day, change);
        } else {
            schedule.overlap.addShift(day, change);
        }
        constraints.add(schedule.overlap);

        return true;

    }

    @Override
    public void execute(Solution solution) {

        super.execute(solution);
        schedule.shifts[day] = replacements.get(day);

    }

}
