package at.ac.tuwien.dbai.ges.solver.algorithm.move.shift;

import at.ac.tuwien.dbai.ges.solver.RandomProvider;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.Move;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.selection.SelectionStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;

import java.util.HashMap;
import java.util.Map;

/**
 * This move changes the start or end of the chosen shift by a small amount.
 *
 * @author Lucas Kletzander
 */
public class ChangeShiftLocal extends Move {

    private final SelectionStrategy strategy;

    private int day;
    private EmployeeSchedule schedule;
    private Map<Integer, Shift> replacements = new HashMap<>();

    public ChangeShiftLocal(Instance instance, SelectionStrategy strategy) {
        super(instance);
        this.strategy = strategy;
    }

    @Override
    public boolean prepare(Solution solution) {

        int abort = 0;

        do {
            day = strategy.selectDay(solution);
            schedule = strategy.selectEmployee(solution);
        } while (schedule.shifts[day] == null && abort++ < 100);

        if (abort >= 100) {
            return false;
        }

        Shift shift = schedule.shifts[day];
        Shift newShift = shift.clone();

        boolean start = RandomProvider.getRandom().nextBoolean();
        boolean earlier = RandomProvider.getRandom().nextBoolean();
        if ((start && shift.startFixed) || (!start && shift.endFixed)) {
            return false;
        }

        if (start) {
            newShift.start += instance.shiftSlotLength * (earlier ? -1 : 1);
        } else {
            newShift.end += instance.shiftSlotLength * (earlier ? -1 : 1);
        }

        if (newShift.start < 0) {
            newShift.start = 0;
        }

        if (newShift.start >= newShift.end) {
            newShift.end = newShift.start + instance.shiftSlotLength;
        }

        replacements.clear();
        replacements.put(day, newShift);

        processShiftConstraints(shift.type, instance.employeeDefinition.getEmployee(schedule.id), day, i -> {
            i.removeShift(schedule.id, shift);
            i.addShift(schedule.id, newShift);
        });
        processShiftArrangementConstraints(instance.employeeDefinition.getEmployee(schedule.id), i -> i.changeRow(schedule.id, schedule.shifts, replacements));

        schedule.overlap.removeShift(day, shift);
        schedule.overlap.addShift(day, newShift);
        constraints.add(schedule.overlap);

        return true;

    }

    @Override
    public void execute(Solution solution) {

        super.execute(solution);
        schedule.shifts[day] = replacements.get(day);

    }

}
