package at.ac.tuwien.dbai.ges.solver.algorithm.move.shift;

import at.ac.tuwien.dbai.ges.solver.RandomProvider;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.Move;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.selection.SelectionStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;

import java.util.HashMap;
import java.util.Map;

/**
 * This move creates a sequence (max length 7) of shifts (with different types or a specific type) or days off.
 *
 * @author Lucas Kletzander
 */
public class CreateSequence extends Move {

    private final SelectionStrategy strategy;
    private final ShiftGenerator generator;

    private EmployeeSchedule schedule;
    private Map<Integer, Shift> replacements = new HashMap<>();

    public CreateSequence(Instance instance, SelectionStrategy strategy) {

        super(instance);
        this.strategy = strategy;
        generator = new ShiftGenerator(instance);

    }

    @Override
    public boolean prepare(Solution solution) {

        int day = strategy.selectDay(solution);
        int length = RandomProvider.getRandom().nextInt(Math.min(7, instance.history + instance.periodLength - day));
        schedule = strategy.selectEmployee(solution);
        int type = RandomProvider.getRandom().nextInt(3);

        replacements.clear();

        Shift shift = null;
        if (type == 1) {
            shift = generator.generate();
        }

        for (int i = day; i <= day + length; i++) {

            if (schedule.fixed[i]) {
                continue;
            }

            if (type == 1) {
                shift = shift.clone();
            } else if (type == 2) {
                shift = generator.generate();
            }

            if (schedule.shifts[i] != null) {

                int finalI = i;
                processShiftConstraints(schedule.shifts[i].type, instance.employeeDefinition.getEmployee(schedule.id), i, j -> j.removeShift(schedule.id, schedule.shifts[finalI]));

                schedule.overlap.removeShift(i, schedule.shifts[i]);

            }
            if (shift != null) {

                Shift finalShift = shift;
                processShiftConstraints(shift.type, instance.employeeDefinition.getEmployee(schedule.id), i, j -> j.addShift(schedule.id, finalShift));

                schedule.overlap.addShift(i, shift);

            }

            replacements.put(i, shift);

        }

        processShiftArrangementConstraints(instance.employeeDefinition.getEmployee(schedule.id), i -> i.changeRow(schedule.id, schedule.shifts, replacements));
        constraints.add(schedule.overlap);

        return true;

    }

    @Override
    public void execute(Solution solution) {

        super.execute(solution);
        replacements.forEach((k, v) -> schedule.shifts[k] = v);

    }

}
