package at.ac.tuwien.dbai.ges.solver.algorithm.move.shift;

import at.ac.tuwien.dbai.ges.solver.RandomProvider;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.definition.ShiftType;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

/**
 * Contains methods to create or change shifts that can be used from different moves.
 *
 * @author Lucas Kletzander
 */
public class ShiftGenerator {

    private final Instance instance;

    public ShiftGenerator(Instance instance) {
        this.instance = instance;
    }

    public Shift generate() {

        Shift shift = new Shift();
        ShiftType type = RandomProvider.getRandomElement(instance.shiftDefinition.shiftTypes.values());
        shift.type = type.id;
        generateShiftStart(shift);
        generateShiftEnd(shift);
        fixShift(shift);
        return shift;

    }

    public boolean changeShift(Shift shift) {

        boolean changeStart = RandomProvider.getRandom().nextBoolean();
        if ((changeStart && shift.startFixed) || (!changeStart && shift.endFixed)) {
            return false;
        }

        if (changeStart) {
            generateShiftStart(shift);
        }

        if (!changeStart || RandomProvider.getRandom().nextBoolean()) {
            generateShiftEnd(shift);
        }

        fixShift(shift);
        return true;

    }

    private void generateShiftStart(Shift shift) {

        ShiftType type = instance.shiftDefinition.shiftTypes.get(shift.type);

        if (instance.shiftDesign) {
            shift.start = RandomProvider.getInBounds(type.earliestStart == -1 ? 0 : type.earliestStart, type.latestStart == -1 ? 24 * 60 - 1 : type.latestStart, instance.shiftSlotLength);
        } else {
            shift.start = type.earliestStart;
        }

    }

    private void generateShiftEnd(Shift shift) {

        ShiftType type = instance.shiftDefinition.shiftTypes.get(shift.type);

        if (instance.shiftDesign) {
            shift.end = RandomProvider.getInBounds(type.earliestEnd == -1 ? shift.start + 1 : type.earliestEnd, type.latestEnd == -1 ? shift.start + 24 * 60 : type.latestEnd, instance.shiftSlotLength);
        } else {
            shift.end = type.latestEnd;
        }

    }

    private void fixShift(Shift shift) {

        if (shift.start >= shift.end) {
            shift.start = shift.end - instance.shiftSlotLength;
        }

        if (shift.start >= 24 * 60) {
            shift.start -= 24 * 60;
            shift.end -= 24 * 60;
        }

    }

}
