package at.ac.tuwien.dbai.ges.solver.algorithm.move.shift;

import at.ac.tuwien.dbai.ges.solver.RandomProvider;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.Move;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.selection.SelectionStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;

import java.util.HashMap;
import java.util.Map;

/**
 * This move swaps to periods of the same length (max 7) within one employee.
 *
 * @author Lucas Kletzander
 */
public class SwapPeriodWithinEmployee extends Move {

    private final SelectionStrategy strategy;

    private EmployeeSchedule schedule;
    private Map<Integer, Shift> replacements = new HashMap<>();

    public SwapPeriodWithinEmployee(Instance instance, SelectionStrategy strategy) {
        super(instance);
        this.strategy = strategy;
    }

    @Override
    public boolean prepare(Solution solution) {

        schedule = strategy.selectEmployee(solution);
        int start1 = strategy.selectDay(solution);
        int start2 = strategy.selectDayExcept(solution, start1);
        int length = RandomProvider.getRandom().nextInt(Math.min(instance.history + instance.periodLength - Math.max(start1, start2), Math.abs(start1 - start2))) % 7;

        replacements.clear();

        for (int i = 0; i <= length; i++) {

            if (schedule.fixed[start1 + i] || schedule.fixed[start2 + i]) {
                continue;
            }

            replacements.put(start1 + i, schedule.shifts[start2 + i]);
            replacements.put(start2 + i, schedule.shifts[start1 + i]);

            processShiftSwap(schedule, schedule, start1 + i, start2 + i);
            processShiftSwap(schedule, schedule, start2 + i, start1 + i);

        }

        processShiftArrangementConstraints(instance.employeeDefinition.getEmployee(schedule.id), i -> i.changeRow(schedule.id, schedule.shifts, replacements));

        return true;

    }

    @Override
    public void execute(Solution solution) {

        super.execute(solution);
        replacements.forEach((k, v) -> schedule.shifts[k] = v);

    }

}
