package at.ac.tuwien.dbai.ges.solver.algorithm.move.shift;

import at.ac.tuwien.dbai.ges.solver.algorithm.move.Move;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.selection.SelectionStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;

import java.util.HashMap;
import java.util.Map;

/**
 * This move swaps two shifts on the same day between different employees.
 *
 * @author Lucas Kletzander
 */
public class SwapShiftsBetweenEmployees extends Move {

    private final SelectionStrategy strategy;

    private int day;
    private EmployeeSchedule schedule1;
    private EmployeeSchedule schedule2;
    private Map<Integer, Shift> replacements1 = new HashMap<>();
    private Map<Integer, Shift> replacements2 = new HashMap<>();

    public SwapShiftsBetweenEmployees(Instance instance, SelectionStrategy strategy) {
        super(instance);
        this.strategy = strategy;
    }

    @Override
    public boolean prepare(Solution solution) {

        if (solution.schedule.size() < 2) {
            return false;
        }

        day = strategy.selectDay(solution);
        schedule1 = strategy.selectEmployee(solution);
        schedule2 = strategy.selectEmployeeExcept(solution, schedule1);

        if (schedule1.fixed[day] || schedule2.fixed[day]) {
            return false;
        }

        replacements1.clear();
        replacements1.put(day, schedule2.shifts[day]);
        replacements2.clear();
        replacements2.put(day, schedule1.shifts[day]);

        processShiftSwap(schedule1, schedule2, day, day);
        processShiftSwap(schedule2, schedule1, day, day);

        processShiftArrangementConstraints(instance.employeeDefinition.getEmployee(schedule1.id), i -> i.changeRow(schedule1.id, schedule1.shifts, replacements1));
        processShiftArrangementConstraints(instance.employeeDefinition.getEmployee(schedule2.id), i -> i.changeRow(schedule2.id, schedule2.shifts, replacements2));

        return true;

    }

    @Override
    public void execute(Solution solution) {

        super.execute(solution);
        schedule1.shifts[day] = replacements1.get(day);
        schedule2.shifts[day] = replacements2.get(day);

    }

}
