package at.ac.tuwien.dbai.ges.solver.algorithm.move.shift;

import at.ac.tuwien.dbai.ges.solver.algorithm.move.Move;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.selection.SelectionStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;

import java.util.HashMap;
import java.util.Map;

/**
 * This move swaps two shifts within the same employee.
 *
 * @author Lucas Kletzander
 */
public class SwapShiftsWithinEmployee extends Move {

    private final SelectionStrategy strategy;

    private int day1;
    private int day2;
    private EmployeeSchedule schedule;
    private Map<Integer, Shift> replacements = new HashMap<>();

    public SwapShiftsWithinEmployee(Instance instance, SelectionStrategy strategy) {
        super(instance);
        this.strategy = strategy;
    }

    @Override
    public boolean prepare(Solution solution) {

        if (instance.periodLength < 2) {
            return false;
        }

        day1 = strategy.selectDay(solution);
        day2 = strategy.selectDayExcept(solution, day1);
        schedule = strategy.selectEmployee(solution);

        if (schedule.fixed[day1] || schedule.fixed[day2]) {
            return false;
        }

        replacements.clear();
        replacements.put(day1, schedule.shifts[day2]);
        replacements.put(day2, schedule.shifts[day1]);

        processShiftSwap(schedule, schedule, day1, day2);
        processShiftSwap(schedule, schedule, day2, day1);

        processShiftArrangementConstraints(instance.employeeDefinition.getEmployee(schedule.id), i -> i.changeRow(schedule.id, schedule.shifts, replacements));

        return true;

    }

    @Override
    public void execute(Solution solution) {

        super.execute(solution);
        schedule.shifts[day1] = replacements.get(day1);
        schedule.shifts[day2] = replacements.get(day2);

    }

}
