package at.ac.tuwien.dbai.ges.solver.algorithm.move.task;

import at.ac.tuwien.dbai.ges.solver.algorithm.move.Move;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.selection.SelectionStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;
import at.ac.tuwien.dbai.ges.solver.solution.TaskInstance;

/**
 * This move selects a task and an employee and assigns the task to the employee if not currently assigned, else removes it.
 *
 * @author Lucas Kletzander
 */
public class AddOrRemoveTaskAssignment extends Move {

    private final SelectionStrategy strategy;

    private TaskInstance task;
    private EmployeeSchedule schedule;

    public AddOrRemoveTaskAssignment(Instance instance, SelectionStrategy strategy) {
        super(instance);
        this.strategy = strategy;
    }

    @Override
    public boolean prepare(Solution solution) {

        if (solution.schedule.isEmpty()) {
            return false;
        }

        task = strategy.selectTask(solution);
        schedule = strategy.selectEmployee(solution);

        if (task.employees.contains(schedule.id)) {
            processTaskConstraints(task.demandId, instance.employeeDefinition.getEmployee(schedule.id).skills, i -> i.removeTask(task));
            task.parts.forEach(i -> schedule.overlap.removeOccupied(task.day, i.start, i.end));
        } else {
            processTaskConstraints(task.demandId, instance.employeeDefinition.getEmployee(schedule.id).skills, i -> i.addTask(task));
            task.parts.forEach(i -> schedule.overlap.addOccupied(task.day, i.start, i.end));
        }

        constraints.add(schedule.overlap);

        return true;

    }

    @Override
    public void execute(Solution solution) {

        super.execute(solution);

        if (task.employees.contains(schedule.id)) {

            task.employees.remove(schedule.id);
            schedule.getTasksAt(task).remove(task.demandId);

        } else {

            task.employees.add(schedule.id);
            schedule.getTasksAt(task).add(task.demandId);

        }

    }

}
