package at.ac.tuwien.dbai.ges.solver.algorithm.move.task;

import at.ac.tuwien.dbai.ges.solver.RandomProvider;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.Move;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.selection.SelectionStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;
import at.ac.tuwien.dbai.ges.solver.solution.TaskInstance;

/**
 * This move changes the assignment of a task from one employee to another employee (or aborts if no assignment is found).
 *
 * @author Lucas Kletzander
 */
public class ChangeTaskAssignment extends Move {

    private final SelectionStrategy strategy;

    private TaskInstance task;
    private EmployeeSchedule schedule1;
    private EmployeeSchedule schedule2;

    public ChangeTaskAssignment(Instance instance, SelectionStrategy strategy) {
        super(instance);
        this.strategy = strategy;
    }

    @Override
    public boolean prepare(Solution solution) {

        TaskInstance task = strategy.selectTask(solution);
        if (task.employees.isEmpty() || solution.schedule.isEmpty() || task.employees.size() == solution.schedule.size()) {
            return false;
        }

        EmployeeSchedule schedule1 = solution.schedule.get(RandomProvider.getRandomElement(task.employees));
        EmployeeSchedule schedule2;
        do {
            schedule2 = solution.scheduleList.get(RandomProvider.getRandom().nextInt(solution.scheduleList.size()));
        } while (schedule2.equals(schedule1) || task.employees.contains(schedule2.id));

        return prepare(solution, schedule1, schedule2, task.demandId);

    }

    public boolean prepare(Solution solution, EmployeeSchedule fromSchedule, EmployeeSchedule toSchedule, String taskId) {

        task = solution.taskInstances.get(taskId);
        schedule1 = fromSchedule;
        schedule2 = toSchedule;

        processTaskConstraints(task.demandId, instance.employeeDefinition.getEmployee(schedule1.id).skills, i -> i.removeTask(task));
        processTaskConstraints(task.demandId, instance.employeeDefinition.getEmployee(schedule2.id).skills, i -> i.addTask(task));

        task.parts.forEach(i -> {
            schedule1.overlap.removeOccupied(task.day, i.start, i.end);
            schedule2.overlap.addOccupied(task.day, i.start, i.end);
        });

        constraints.add(schedule1.overlap);
        constraints.add(schedule2.overlap);

        return true;

    }

    @Override
    public void execute(Solution solution) {

        super.execute(solution);

        task.employees.remove(schedule1.id);
        schedule1.getTasksAt(task).remove(task.demandId);

        task.employees.add(schedule2.id);
        schedule2.getTasksAt(task).add(task.demandId);

    }

}
