package at.ac.tuwien.dbai.ges.solver.algorithm.move.task;

import at.ac.tuwien.dbai.ges.solver.RandomProvider;
import at.ac.tuwien.dbai.ges.solver.algorithm.Algorithm;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;

/**
 * This algorithms randomly tries to reduce shift lengths without harming task assignments to those shifts.
 *
 * @author Lucas Kletzander
 */
public class MinimizeShifts extends Algorithm {

    private final ReduceShiftLength reduceShiftLength;

    public MinimizeShifts(Instance instance) {
        reduceShiftLength = new ReduceShiftLength(instance);
    }

    @Override
    public void apply(Instance instance, Solution solution) {

        for (EmployeeSchedule schedule : solution.schedule.values()) {

            for (int i = instance.history; i < schedule.shifts.length; i++) {

                if (schedule.shifts[i] != null) {

                    int failCount = 0;

                    while (failCount < 10) {

                        if (!reduceShiftLength.prepare(schedule, i, RandomProvider.getRandom().nextBoolean(), RandomProvider.getRandom().nextInt(schedule.shifts[i].getDuration() / instance.shiftSlotLength))) {
                            failCount++;
                            continue;
                        }

                        double value = reduceShiftLength.evaluate();

                        if (value > 0) {
                            reduceShiftLength.abort();
                            failCount++;
                            continue;
                        }

                        solution.value += value;
                        reduceShiftLength.execute(solution);

                    }

                }

            }

        }

    }

}
