package at.ac.tuwien.dbai.ges.solver.algorithm.move.task;

import at.ac.tuwien.dbai.ges.solver.RandomProvider;
import at.ac.tuwien.dbai.ges.solver.algorithm.move.Move;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.EmployeeSchedule;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;

import java.util.HashMap;
import java.util.Map;

/**
 * This move reduces the length of a shift by changing start or end time (or aborts if that is not possible).
 *
 * @author Lucas Kletzander
 */
public class ReduceShiftLength extends Move {

    private EmployeeSchedule schedule;
    private int day;
    private Map<Integer, Shift> replacements = new HashMap<>();

    public ReduceShiftLength(Instance instance) {
        super(instance);
    }

    @Override
    public boolean prepare(Solution solution) {

        EmployeeSchedule schedule = solution.scheduleList.get(RandomProvider.getRandom().nextInt(solution.scheduleList.size()));

        int day;
        int count = 0;

        do {
            day = RandomProvider.getRandom().nextInt(instance.periodLength);
            count++;
        } while (schedule.shifts[day] == null && count < 10);

        return count < 10 && prepare(schedule, day, RandomProvider.getRandom().nextBoolean(), RandomProvider.getRandom().nextInt(schedule.shifts[day].getDuration() / instance.shiftSlotLength));

    }

    public boolean prepare(EmployeeSchedule schedule, int day, boolean start, int numSlots) {

        this.schedule = schedule;
        this.day = day;

        Shift shift = schedule.shifts[day];
        if (shift.start + instance.shiftSlotLength >= shift.end || (start && shift.startFixed) || (!start && shift.endFixed)) {
            return false;
        }

        Shift newShift = shift.clone();

        if (start) {
            newShift.start += numSlots * instance.shiftSlotLength;
        } else {
            newShift.end -= numSlots * instance.shiftSlotLength;
        }

        replacements.clear();
        replacements.put(day, newShift);

        processShiftConstraints(shift.type, instance.employeeDefinition.getEmployee(schedule.id), day, i -> {
            i.removeShift(schedule.id, shift);
            i.addShift(schedule.id, newShift);
        });
        processShiftArrangementConstraints(instance.employeeDefinition.getEmployee(schedule.id), i -> i.changeRow(schedule.id, schedule.shifts, replacements));

        schedule.overlap.removeShift(day, shift);
        schedule.overlap.addShift(day, newShift);
        constraints.add(schedule.overlap);

        return true;

    }

    @Override
    public void execute(Solution solution) {

        super.execute(solution);
        schedule.shifts[day] = replacements.get(day);

    }

}
