package at.ac.tuwien.dbai.ges.solver.constraint.breaks;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.BreakArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Break;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.NavigableMap;

/**
 * Constrains the fraction of a shift that is spent on break.
 *
 * @author Lucas Kletzander
 */
public class BreakTimeFractionConstraint extends BreakArrangementConstraint {

    public WeightStrategy<Double> weightStrategy;

    public BreakTimeFractionConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeShift(Shift shift, NavigableMap<Integer, Break> breaks, NavigableMap<Integer, Break> replacements) {

        int time = shift.breakTime;
        newValue -= weightStrategy.evaluate(((double) time) / shift.getDuration());

        for (Integer pos : replacements.keySet()) {

            if (breaks.get(pos) != null) {
                time -= breaks.get(pos).getDuration();
            }

            if (replacements.get(pos) != null) {
                time += replacements.get(pos).getDuration();
            }

        }

        newValue += weightStrategy.evaluate(((double) time) / shift.getDuration());

    }

    @Override
    public void removeShift(Shift shift) {
        newValue -= weightStrategy.evaluate(((double) shift.breakTime) / shift.getDuration());
    }

    @Override
    public void addShift(Shift shift) {
        newValue += weightStrategy.evaluate(((double) shift.breakTime) / shift.getDuration());
    }

}
