package at.ac.tuwien.dbai.ges.solver.constraint.breaks;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.BreakArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Break;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.ArrayList;
import java.util.List;
import java.util.NavigableMap;

/**
 * Constrains the start time of the break relative to the shift start.
 *
 * @author Lucas Kletzander
 */
public class StartShiftConstraint extends BreakArrangementConstraint {

    public String breakType;
    public List<WeightStrategy<Integer>> weightStrategy = new ArrayList<>();

    public StartShiftConstraint(Instance instance, String breakType) {
        super(instance);
        this.breakType = breakType;
    }

    @Override
    public void changeShift(Shift shift, NavigableMap<Integer, Break> breaks, NavigableMap<Integer, Break> replacements) {

        for (Integer pos : replacements.keySet()) {

            if (breaks.get(pos) != null && breaks.get(pos).id.equals(breakType)) {
                newValue -= weightStrategy.stream().mapToDouble(i -> i.evaluate(breaks.get(pos).start - shift.start)).sum();
            }

            if (replacements.get(pos) != null && replacements.get(pos).id.equals(breakType)) {
                newValue += weightStrategy.stream().mapToDouble(i -> i.evaluate(replacements.get(pos).start - shift.start)).sum();
            }

        }

    }

}
