package at.ac.tuwien.dbai.ges.solver.constraint.breaks;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.BreakArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Break;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.NavigableMap;

/**
 * Constrains the work time after a break.
 *
 * @author Lucas Kletzander
 */
public class WorkAfterConstraint extends BreakArrangementConstraint {

    public String breakType;
    public List<WeightStrategy<Integer>> weightStrategy = new ArrayList<>();

    public WorkAfterConstraint(Instance instance, String breakType) {
        super(instance);
        this.breakType = breakType;
    }

    @Override
    public void changeShift(Shift shift, NavigableMap<Integer, Break> breaks, NavigableMap<Integer, Break> replacements) {

        for (Integer pos : replacements.keySet()) {

            if (breaks.get(pos) != null && breaks.get(pos).id.equals(breakType)) {
                Map.Entry<Integer, Break> next = breaks.higherEntry(pos);
                newValue -= weightStrategy.stream().mapToDouble(i -> i.evaluate((next == null ? shift.end : next.getValue().start) - breaks.get(pos).end)).sum();
            }

            if (replacements.get(pos) != null && replacements.get(pos).id.equals(breakType)) {
                Break next = getNext(shift, replacements, pos);
                newValue += weightStrategy.stream().mapToDouble(i -> i.evaluate((next == null ? shift.end : next.start) - replacements.get(pos).end)).sum();
            }

        }

    }

    private Break getNext(Shift shift, NavigableMap<Integer, Break> replacements, int pos) {

        Integer oldNext = shift.breaks.higherKey(pos);
        Integer newNext = pos;
        do {
            newNext = replacements.higherKey(newNext);
        } while (newNext != null && replacements.get(newNext) == null);

        if (oldNext == null) {
            return newNext == null ? null : replacements.get(newNext);
        } else {
            return newNext == null || oldNext < newNext ? shift.breaks.get(oldNext) : replacements.get(newNext);
        }

    }

}
