package at.ac.tuwien.dbai.ges.solver.constraint.breaks;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.BreakArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Break;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.NavigableMap;

/**
 * Constrains the work time after the last break.
 *
 * @author Lucas Kletzander
 */
public class WorkAfterLastConstraint extends BreakArrangementConstraint {

    public WeightStrategy<Integer> weightStrategy;

    public WorkAfterLastConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeShift(Shift shift, NavigableMap<Integer, Break> breaks, NavigableMap<Integer, Break> replacements) {

        if (breaks.isEmpty()) {
            newValue += weightStrategy.evaluate(shift.end - replacements.lastEntry().getValue().end);
            return;
        }

        if (replacements.lastKey() < breaks.lastKey()) {
            return;
        }

        int diff;
        if (replacements.lastKey() > breaks.lastKey()) {
            diff = shift.end - replacements.lastEntry().getValue().end;
        } else {
            int newPos = Math.min(breaks.descendingKeySet().stream().filter(i -> !replacements.containsKey(i) || replacements.get(i) != null).findFirst().orElse(-1),
                    replacements.descendingKeySet().stream().filter(i -> replacements.get(i) != null).findFirst().orElse(-1));
            diff = newPos == -1 ? -1 : shift.end - replacements.getOrDefault(newPos, breaks.get(newPos)).end;
        }

        newValue -= weightStrategy.evaluate(shift.end - breaks.lastEntry().getValue().end);
        if (diff > -1) {
            newValue += weightStrategy.evaluate(diff);
        }

    }

    @Override
    public void removeShift(Shift shift) {
        if (!shift.breaks.isEmpty()) {
            newValue -= weightStrategy.evaluate(shift.end - shift.breaks.lastEntry().getValue().end);
        }
    }

    @Override
    public void addShift(Shift shift) {
        if (!shift.breaks.isEmpty()) {
            newValue += weightStrategy.evaluate(shift.end - shift.breaks.lastEntry().getValue().end);
        }
    }

}
