package at.ac.tuwien.dbai.ges.solver.constraint.breaks;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.BreakArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Break;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.NavigableMap;

/**
 * Constrains the work time before a break.
 *
 * @author Lucas Kletzander
 */
public class WorkBeforeConstraint extends BreakArrangementConstraint {

    public String breakType;
    public List<WeightStrategy<Integer>> weightStrategy = new ArrayList<>();

    public WorkBeforeConstraint(Instance instance, String breakType) {
        super(instance);
        this.breakType = breakType;
    }

    @Override
    public void changeShift(Shift shift, NavigableMap<Integer, Break> breaks, NavigableMap<Integer, Break> replacements) {

        for (Integer pos : replacements.keySet()) {

            if (breaks.get(pos) != null && breaks.get(pos).id.equals(breakType)) {
                Map.Entry<Integer, Break> prev = breaks.lowerEntry(pos);
                newValue -= weightStrategy.stream().mapToDouble(i -> i.evaluate(breaks.get(pos).start - (prev == null ? shift.start : prev.getValue().end))).sum();
            }

            if (replacements.get(pos) != null && replacements.get(pos).id.equals(breakType)) {
                Break prev = getPrevious(shift, replacements, pos);
                newValue += weightStrategy.stream().mapToDouble(i -> i.evaluate(replacements.get(pos).start - (prev == null ? shift.start : prev.end))).sum();
            }

        }

    }

    private Break getPrevious(Shift shift, NavigableMap<Integer, Break> replacements, int pos) {

        Integer oldPrev = shift.breaks.lowerKey(pos);
        Integer newPrev = pos;
        do {
            newPrev = replacements.lowerKey(newPrev);
        } while (newPrev != null && replacements.get(newPrev) == null);

        if (oldPrev == null) {
            return newPrev == null ? null : replacements.get(newPrev);
        } else {
            return newPrev == null || oldPrev > newPrev ? shift.breaks.get(oldPrev) : replacements.get(newPrev);
        }

    }

}
