package at.ac.tuwien.dbai.ges.solver.constraint.breaks;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.BreakArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Break;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.NavigableMap;

/**
 * Constrains the work time before the first break.
 *
 * @author Lucas Kletzander
 */
public class WorkBeforeFirstConstraint extends BreakArrangementConstraint {

    public WeightStrategy<Integer> weightStrategy;

    public WorkBeforeFirstConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeShift(Shift shift, NavigableMap<Integer, Break> breaks, NavigableMap<Integer, Break> replacements) {

        if (breaks.isEmpty()) {
            newValue += weightStrategy.evaluate(replacements.firstEntry().getValue().start - shift.start);
            return;
        }

        if (replacements.firstKey() > breaks.firstKey()) {
            return;
        }

        int diff;
        if (replacements.firstKey() < breaks.firstKey()) {
            diff = replacements.firstEntry().getValue().start - shift.start;
        } else {
            int newPos = Math.min(breaks.keySet().stream().filter(i -> !replacements.containsKey(i) || replacements.get(i) != null).findFirst().orElse(Integer.MAX_VALUE),
                    replacements.keySet().stream().filter(i -> replacements.get(i) != null).findFirst().orElse(Integer.MAX_VALUE));
            diff = newPos == Integer.MAX_VALUE ? Integer.MAX_VALUE : replacements.getOrDefault(newPos, breaks.get(newPos)).start - shift.start;
        }

        newValue -= weightStrategy.evaluate(breaks.firstEntry().getValue().start - shift.start);
        if (diff < Integer.MAX_VALUE) {
            newValue += weightStrategy.evaluate(diff);
        }

    }

    @Override
    public void removeShift(Shift shift) {
        if (!shift.breaks.isEmpty()) {
            newValue -= weightStrategy.evaluate(shift.breaks.firstEntry().getValue().start - shift.start);
        }
    }

    @Override
    public void addShift(Shift shift) {
        if (!shift.breaks.isEmpty()) {
            newValue += weightStrategy.evaluate(shift.breaks.firstEntry().getValue().start - shift.start);
        }
    }

}
