package at.ac.tuwien.dbai.ges.solver.constraint.employee;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.EmployeeConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;

import java.util.List;

/**
 * Constrains the number of variable employees.
 *
 * @author Lucas Kletzander
 */
public class VariableEmployeeConstraint extends EmployeeConstraint {//TODO each instance of a variable employee needs their own unique id

    public String match;
    public int count;
    public int newCount;
    public int countOther;
    public int newCountOther;
    public List<WeightStrategy<Integer>> countWeights;
    public List<WeightStrategy<Double>> fractionWeights;

    public VariableEmployeeConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void addEmployee(String id) {

        if (match.equals(id)) {
            newCount++;
        } else {
            newCountOther++;
        }

    }

    @Override
    public void removeEmployee(String id) {

        if (match.equals(id)) {
            newCount--;
        } else {
            newCountOther--;
        }

    }

    @Override
    public double evaluate() {
        newValue = countWeights.stream().mapToDouble(i -> i.evaluate(count)).sum() + fractionWeights.stream().mapToDouble(i -> i.evaluate(((double) newCount) / (newCount + newCountOther))).sum();
        return super.evaluate();
    }

    @Override
    public void execute() {
        super.execute();
        count = newCount;
        countOther = newCountOther;
    }

    @Override
    public void abort() {
        super.abort();
        newCount = count;
        newCountOther = countOther;
    }

}
