package at.ac.tuwien.dbai.ges.solver.constraint.hierarchy;

import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.Map;
import java.util.function.Predicate;

/**
 * A constraint where the evaluation depends on the previous and following shifts.
 *
 * @author Lucas Kletzander
 */
public abstract class ShiftArrangementConstraint extends Constraint {

    public ShiftArrangementConstraint(Instance instance) {
        super(instance);
    }

    public abstract void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements);

    public void initRow(String id, Shift[] row, Map<Integer, Shift> replacements) {
        changeRow(id, new Shift[row.length], replacements);
    }

    public int getPrevious(Shift[] row, int pos, Predicate<Shift> filter) {

        for (pos--; pos >= 0; pos--) {
            if (filter.test(row[pos])) {
                return pos;
            }
        }

        return pos;

    }

    public int getPrevious(Shift[] row, Map<Integer, Shift> replacements, int pos, Predicate<Shift> filter) {

        for (pos--; pos >= 0; pos--) {
            if (filter.test(replacements.getOrDefault(pos, row[pos]))) {
                return pos;
            }
        }

        return pos;

    }

    public int getNext(Shift[] row, int pos, Predicate<Shift> filter) {

        for (pos++; pos < row.length; pos++) {
            if (filter.test(row[pos])) {
                return pos;
            }
        }

        return pos;

    }

    public int getNext(Shift[] row, Map<Integer, Shift> replacements, int pos, Predicate<Shift> filter) {

        for (pos++; pos < row.length; pos++) {
            if (filter.test(replacements.getOrDefault(pos, row[pos]))) {
                return pos;
            }
        }

        return pos;

    }

    public boolean checkCutoff(int startExclusive, int endExclusive) {
        return (startExclusive < 0 && instance.cutoffStart) || (endExclusive >= instance.history + instance.periodLength && instance.cutoffEnd);
    }

}
