package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.List;

/**
 * Constrains the average shift length.
 *
 * @author Lucas Kletzander
 */
public class AverageShiftLengthConstraint extends ShiftConstraint {

    public double avg;
    public int count;
    public double newAvg;
    public int newCount;
    public List<WeightStrategy<Double>> weightStrategy;

    public AverageShiftLengthConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public double evaluate() {
        newValue = weightStrategy.stream().mapToDouble(i -> i.evaluate(newAvg)).sum();
        return super.evaluate();
    }

    @Override
    public void execute() {
        super.execute();
        avg = newAvg;
        count = newCount;
    }

    @Override
    public void abort() {
        super.abort();
        newAvg = avg;
        newCount = count;
    }

    @Override
    public void addShift(String id, Shift shift) {

        if (applies(shift)) {
            newAvg = (newCount * newAvg + shift.getDuration()) / (++newCount);
        }

    }

    @Override
    public void removeShift(String id, Shift shift) {

        if (applies(shift)) {
            newAvg = (newCount * newAvg - shift.getDuration()) / (--newCount);
        }

    }

}
