package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.List;
import java.util.Map;
import java.util.function.Predicate;

/**
 * Constrains forbidden sequences.
 *
 * @author Lucas Kletzander
 */
public class ForbiddenSequenceConstraint extends ShiftArrangementConstraint {

    public List<Predicate<Shift>> pattern;
    public int count;
    public int newCount;
    public WeightStrategy<Integer> weightStrategy;

    public ForbiddenSequenceConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        for (Integer i : replacements.keySet()) {

            for (int j = Math.max(i - pattern.size() + 1, 0); j <= i; j++) {
                if (applies(j) && matches(row, replacements, j, i)) {
                    newCount--;
                }
                if (applies(j) && matches(row, replacements, j, i + 1)) {
                    newCount++;
                }
            }

        }

    }

    public boolean matches(Shift[] row, Map<Integer, Shift> replacements, int pos, int current) {

        if (pos + pattern.size() > row.length) {
            return false;
        }

        for (Predicate<Shift> filter : pattern) {
            if (!filter.test(pos < current ? replacements.getOrDefault(pos, row[pos]) : row[pos])) {
                return false;
            }
            pos++;
        }

        return true;

    }

    @Override
    public double evaluate() {
        newValue = weightStrategy.evaluate(newCount);
        return super.evaluate();
    }

    @Override
    public void execute() {
        super.execute();
        count = newCount;
    }

    @Override
    public void abort() {
        super.abort();
        newCount = count;
    }

}
