package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Predicate;

/**
 * Constrains identical sequences.
 *
 * @author Lucas Kletzander
 */
public class IdenticalSequenceConstraint extends ShiftArrangementConstraint {

    public List<WeightStrategy<Integer>> weightStrategy;

    public IdenticalSequenceConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        for (Integer pos : replacements.keySet()) {

            int startOld = getPrevious(row, replacements, pos, getComparator(row[pos]));
            int endOld = getNext(row, pos, getComparator(row[pos]));
            int startNew = getPrevious(row, replacements, pos, getComparator(replacements.get(pos)));
            int endNew = getNext(row, pos, getComparator(replacements.get(pos)));

            applyEvaluation(startOld, endOld, -1);

            if (startOld < startNew) {
                applyEvaluation(startOld, pos, 1);
            } else if (startNew < startOld) {
                applyEvaluation(startNew, pos, -1);
            }

            if (endOld < endNew) {
                applyEvaluation(pos, endNew, -1);
            } else if (endNew < endOld) {
                applyEvaluation(pos, endOld, 1);
            }

            applyEvaluation(startNew, endNew, 1);

        }

    }

    @Override
    public void initRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        applyEvaluation(-1, replacements.size(), 1);
        super.initRow(id, row, replacements);

    }

    private Predicate<Shift> getComparator(Shift shift) {
        return shift == null ? Objects::nonNull : i -> i == null || !i.type.equals(shift.type);
    }

    private void applyEvaluation(int startExclusive, int endExclusive, double factor) {

        while (startExclusive < endExclusive - 1 && !applies(startExclusive + 1)) {
            startExclusive++;
        }

        if (startExclusive < endExclusive - 1) {
            int finalStartExclusive = startExclusive;
            newValue += weightStrategy.stream().mapToDouble(i -> i.evaluate(endExclusive - finalStartExclusive - 1)).sum() * factor;
        }

    }

}
