package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.HashMap;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

/**
 * Constrains the gap in workload between employees.
 *
 * @author Lucas Kletzander
 */
public class LargestWorkloadGapConstraint extends ShiftConstraint {

    public SortedMap<Integer, Integer> workloadValues = new TreeMap<>();
    public Map<String, Integer> values = new HashMap<>();
    public Map<String, Integer> newValues = new HashMap<>();
    public WeightStrategy<Integer> weightStrategy;

    public LargestWorkloadGapConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void addShift(String id, Shift shift) {

        Integer oldValue = newValues.getOrDefault(id, values.getOrDefault(id, 0));
        Integer newValue = oldValue + shift.end - shift.start;
        updateWorkloadValues(oldValue, newValue);
        newValues.put(id, newValue);

    }

    @Override
    public void removeShift(String id, Shift shift) {

        Integer oldValue = newValues.getOrDefault(id, values.getOrDefault(id, 0));
        Integer newValue = oldValue - shift.end + shift.start;
        updateWorkloadValues(oldValue, newValue);
        newValues.put(id, newValue);

    }

    private void updateWorkloadValues(Integer oldWorkload, Integer newWorkload) {

        if (oldWorkload != 0) {
            Integer oldCount = workloadValues.get(oldWorkload);
            if (oldCount == 1) {
                workloadValues.remove(oldWorkload);
            } else {
                workloadValues.put(oldWorkload, oldCount - 1);
            }
        }

        if (newWorkload != 0) {
            workloadValues.put(newWorkload, workloadValues.getOrDefault(newWorkload, 0) + 1);
        }

    }

    @Override
    public double evaluate() {
        newValue = weightStrategy.evaluate(workloadValues.lastKey() - workloadValues.firstKey());
        return super.evaluate();
    }

    @Override
    public void execute() {
        super.execute();
        values.putAll(newValues);
        newValues.clear();
    }

    @Override
    public void abort() {
        super.abort();
        newValues.forEach((i, j) -> updateWorkloadValues(j, values.get(i)));
        newValues.clear();
    }

}
