package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.Util;
import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.Map;
import java.util.Objects;

/**
 * Constrains the minimum rest time.
 *
 * @author Lucas Kletzander
 */
public class MinRestTimeConstraint extends ShiftArrangementConstraint {

    public WeightStrategy<Integer> weightStrategy;

    public MinRestTimeConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        for (Integer i : replacements.keySet()) {

            int prev = getPrevious(row, replacements, i, Objects::nonNull);
            int next = getNext(row, i, Objects::nonNull);

            if (row[i] == null) {
                if (prev >= 0 && next < row.length) {
                    newValue -= weightStrategy.evaluate(Util.getTimeDifference(prev, replacements.getOrDefault(prev, row[prev]).end, next, row[next].start));
                }
            } else {
                if (prev >= 0) {
                    newValue -= weightStrategy.evaluate(Util.getTimeDifference(prev, replacements.getOrDefault(prev, row[prev]).end, i, row[i].start));
                }
                if (next < row.length) {
                    newValue -= weightStrategy.evaluate(Util.getTimeDifference(i, row[i].end, next, row[next].start));
                }
            }

            if (replacements.get(i) == null) {
                if (prev >= 0 && next < row.length) {
                    newValue += weightStrategy.evaluate(Util.getTimeDifference(prev, replacements.getOrDefault(prev, row[prev]).end, next, row[next].start));
                }
            } else {
                if (prev >= 0) {
                    newValue += weightStrategy.evaluate(Util.getTimeDifference(prev, replacements.getOrDefault(prev, row[prev]).end, i, replacements.get(i).start));
                }
                if (next < row.length) {
                    newValue += weightStrategy.evaluate(Util.getTimeDifference(i, replacements.get(i).end, next, row[next].start));
                }
            }

        }

    }

}
