package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.Map;

/**
 * Constrains areas without shifts.
 *
 * @author Lucas Kletzander
 */
public class NoShiftConstraint extends ShiftArrangementConstraint {

    public int startDay = 0;
    public int endDay = Integer.MAX_VALUE;
    public int startTime = -1;
    public int endTime = -1;
    public int count;
    public int newCount;
    public WeightStrategy<Integer> strategy;

    public NoShiftConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        for (int position : replacements.keySet()) {

            if (startTime == -1 && endTime == -1) {

                if (startDay <= position && endDay >= position) {

                    if (row[position] != null) {
                        newCount--;
                    }

                    if (replacements.get(position) != null) {
                        newCount++;
                    }

                }

            } else {

                if (row[position] != null && instance.compare(startDay, startTime, position, row[position].end) <= 0 && instance.compare(endDay, endTime, position, row[position].start) >= 0) {
                    newCount--;
                }

                if (replacements.get(position) != null && instance.compare(startDay, startTime, position, replacements.get(position).end) <= 0 && instance.compare(endDay, endTime, position, replacements.get(position).start) >= 0) {
                    newCount++;
                }

            }

        }

    }

    @Override
    public double evaluate() {
        newValue = strategy.evaluate(newCount);
        return super.evaluate();
    }

    @Override
    public void execute() {
        super.execute();
        count = newCount;
    }

    @Override
    public void abort() {
        super.abort();
        newCount = count;
    }
}
