package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;

/**
 * Counts the number of shifts.
 *
 * @author Lucas Kletzander
 */
public class ShiftCountConstraint extends ShiftArrangementConstraint {

    public Map<String, Integer> count = new HashMap<>();
    public Map<String, Integer> newCount = new HashMap<>();
    public Predicate<Shift> filter;
    public List<WeightStrategy<Integer>> weightStrategy;

    public ShiftCountConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        if (count.containsKey(id)) {
            newValue -= weightStrategy.stream().mapToDouble(i -> i.evaluate(count.get(id))).sum();
        }

        int cur = count.getOrDefault(id, 0);

        for (Integer i : replacements.keySet()) {

            if (!applies(i)) {
                continue;
            }

            if (count.containsKey(id) && filter.test(row[i])) {
                cur--;
            }
            if (filter.test(replacements.get(i))) {
                cur++;
            }

        }

        newCount.put(id, cur);
        newValue += weightStrategy.stream().mapToDouble(i -> i.evaluate(newCount.get(id))).sum();

    }

    @Override
    public void execute() {
        super.execute();
        count.putAll(newCount);
        newCount.clear();
    }

    @Override
    public void abort() {
        super.abort();
        newCount.clear();
    }

}
