package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.List;
import java.util.Map;

/**
 * Constrains shift end times.
 *
 * @author Lucas Kletzander
 */
public class ShiftEndTimesConstraint extends ShiftArrangementConstraint {

    public List<WeightStrategy<Integer>> weightStrategy;

    public ShiftEndTimesConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        for (Integer pos : replacements.keySet()) {

            if (!applies(pos)) {
                continue;
            }

            if (row[pos] != null && applies(row[pos])) {
                newValue -= weightStrategy.stream().mapToDouble(i -> i.evaluate(row[pos].end)).sum();
            }

            if (replacements.get(pos) != null && applies(replacements.get(pos))) {
                newValue += weightStrategy.stream().mapToDouble(i -> i.evaluate(replacements.get(pos).end)).sum();
            }

        }

    }

}
