package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Constrains the number of shift instances.
 *
 * @author Lucas Kletzander
 */
public class ShiftInstanceConstraint extends ShiftConstraint {

    public Map<Integer, Map<Integer, Integer>> map = new HashMap<>();
    public LinkedList<Change> changes = new LinkedList<>();
    public int count;
    public int newCount;
    public List<WeightStrategy<Integer>> weightStrategy;

    public ShiftInstanceConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public double evaluate() {
        newValue = weightStrategy.stream().mapToDouble(i -> i.evaluate(newCount)).sum();
        return super.evaluate();
    }

    @Override
    public void execute() {
        super.execute();
        count = newCount;
        changes.clear();
    }

    @Override
    public void abort() {
        super.abort();
        newCount = count;
        changes.forEach(i -> map.get(i.start).put(i.end, i.count));
        changes.clear();
    }

    @Override
    public void addShift(String id, Shift shift) {

        if (applies(shift)) {
            Map<Integer, Integer> startMap = map.computeIfAbsent(shift.start, i -> new HashMap<>());
            Integer current = startMap.merge(shift.end, 1, (i, j) -> i + j);
            if (current == 1) {
                newCount++;
            }
            changes.addFirst(new Change(shift.start, shift.end, current - 1));
        }

    }

    @Override
    public void removeShift(String id, Shift shift) {

        if (applies(shift)) {
            Integer current = map.get(shift.start).compute(shift.end, (k, v) -> v - 1);
            if (current == 0) {
                newCount--;
            }
            changes.addFirst(new Change(shift.start, shift.end, current + 1));
        }

    }

    public class Change {

        public int start;
        public int end;
        public int count;

        public Change(int start, int end, int count) {
            this.start = start;
            this.end = end;
            this.count = count;
        }

    }

}
