package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.List;
import java.util.Map;
import java.util.function.Predicate;
import java.util.stream.IntStream;

/**
 * Constrains pairs of shifts.
 *
 * @author Lucas Kletzander
 */
public class ShiftPairConstraint extends ShiftArrangementConstraint {

    public int[] count;
    public Map<Integer, Integer> countUpdate;
    public Map<String, List<List<Predicate<Shift>>>> filter;
    public double weight;
    public boolean pair;

    public ShiftPairConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        for (Integer i : replacements.keySet()) {

            for (List<Predicate<Shift>> list : filter.get(id)) {

                for (int j = Math.max(0, i - list.size() + 1); j <= i; j++) {

                    if (!applies(j)) {
                        continue;
                    }

                    int cur = countUpdate.getOrDefault(j, count[j]);
                    int newCur = cur;

                    int finalJ = j;
                    if (!IntStream.range(0, list.size()).filter(k -> !list.get(k).test(row[finalJ + k])).findFirst().isPresent()) {
                        newCur--;
                    }

                    if (!IntStream.range(0, list.size()).filter(k -> !list.get(k).test(replacements.getOrDefault(finalJ + k, row[finalJ + k]))).findFirst().isPresent()) {
                        newCur++;
                    }

                    updateValue(cur, newCur);
                    countUpdate.put(j, newCur);

                }

            }

        }

    }

    private void updateValue(int oldCount, int newCount) {

        if (pair) {

            if (oldCount != 0 && oldCount != filter.size()) {
                newValue--;
            }
            if (newCount != 0 && newCount != filter.size()) {
                newValue++;
            }

        } else {

            if (oldCount == filter.size()) {
                newValue--;
            }
            if (newCount == filter.size()) {
                newValue++;
            }

        }

    }

    @Override
    public void execute() {
        super.execute();
        countUpdate.forEach((i, j) -> count[i] = j);
        countUpdate.clear();
    }

    @Override
    public void abort() {
        super.abort();
        countUpdate.clear();
    }

}
