package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.Map;
import java.util.function.Predicate;

/**
 * Constrains shift requests.
 *
 * @author Lucas Kletzander
 */
public class ShiftRequestConstraint extends ShiftArrangementConstraint {

    public int count = 0;
    public int newCount = 0;
    public Predicate<Shift> filter;
    public double weight;
    public boolean onRequest;

    public ShiftRequestConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        for (Integer i : replacements.keySet()) {

            if (!applies(i)) {
                continue;
            }

            if (filter.test(row[i])) {
                newCount--;
            }
            if (filter.test(replacements.get(i))) {
                newCount++;
            }

        }

    }

    @Override
    public double evaluate() {
        newValue = onRequest == (newCount == 0) ? weight : 0;
        return super.evaluate();
    }

    @Override
    public void execute() {
        super.execute();
        count = newCount;
    }

    @Override
    public void abort() {
        super.abort();
        newCount = count;
    }

}
