package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.ArrayList;
import java.util.List;

/**
 * Constrains shift workforce requirements.
 *
 * @author Lucas Kletzander
 */
public class ShiftRequirementConstraint extends ShiftConstraint {

    public int count;
    public int newCount;
    public List<WeightStrategy<Integer>> weightStrategy = new ArrayList<>();

    public ShiftRequirementConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void addShift(String id, Shift shift) {
        newCount++;
    }

    @Override
    public void removeShift(String id, Shift shift) {
        newCount--;
    }

    @Override
    public double evaluate() {
        newValue = weightStrategy.stream().mapToDouble(i -> i.evaluate(newCount)).sum();
        return super.evaluate();
    }

    @Override
    public void execute() {
        super.execute();
        count = newCount;
    }

    @Override
    public void abort() {
        super.abort();
        newCount = count;
    }

    @Override
    public ShiftRequirementConstraint clone() {

        ShiftRequirementConstraint constraint = new ShiftRequirementConstraint(instance);
        constraint.weightStrategy = weightStrategy;
        return constraint;

    }

}
