package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;

/**
 * Constrains shift sequences.
 *
 * @author Lucas Kletzander
 */
public class ShiftSequenceConstraint extends ShiftArrangementConstraint {

    public Predicate<Shift> filter;
    public List<WeightStrategy<Integer>> weightStrategy;
    private Map<Integer, Integer> minChanges = new HashMap<>();
    private Map<Integer, Integer> maxChanges = new HashMap<>();

    public ShiftSequenceConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        for (Integer pos : replacements.keySet()) {

            if (filter.test(row[pos]) == filter.test(replacements.get(pos))) {
                continue;
            }

            int endOld = getNext(row, pos, filter.negate());
            int startNew = getPrevious(row, replacements, pos, filter.negate());

            if (filter.test(row[pos])) {
                applyEvaluation(startNew, endOld, -1);
                applyEvaluation(startNew, pos, 1);
                applyEvaluation(pos, endOld, 1);
            } else {
                applyEvaluation(startNew, pos, -1);
                applyEvaluation(pos, endOld, -1);
                applyEvaluation(startNew, endOld, 1);
            }

        }

        applyChanges();

    }


    @Override
    public void initRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        if (filter.test(null)) {
            applyEvaluation(-1, replacements.size(), 1);
        }

        super.initRow(id, row, replacements);

    }

    private void applyEvaluation(int startExclusive, int endExclusive, int factor) {

        if (applies(startExclusive + 1, endExclusive - 1) && startExclusive < endExclusive - 1) {
            if (!checkCutoff(startExclusive, endExclusive)) {
                minChanges.merge(endExclusive - startExclusive - 1, factor, (i, j) -> i + j);
            }
            maxChanges.merge(endExclusive - startExclusive - 1, factor, (i, j) -> i + j);
        }

    }

    private void applyChanges() {

        minChanges.forEach((k, v) -> newValue += (v == 0 ? 0 : weightStrategy.stream().filter(i -> i.type == WeightStrategy.Type.MIN).mapToDouble(i -> i.evaluate(k)).sum() * v));
        maxChanges.forEach((k, v) -> newValue += (v == 0 ? 0 : weightStrategy.stream().filter(i -> i.type == WeightStrategy.Type.MAX).mapToDouble(i -> i.evaluate(k)).sum() * v));
        minChanges.clear();
        maxChanges.clear();

    }

}
