package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.HashSet;
import java.util.Set;

/**
 * Constrains valid shift types.
 *
 * @author Lucas Kletzander
 */
public class ValidShiftTypeConstraint extends ShiftConstraint {

    public Set<String> validTypes = new HashSet<>();
    public WeightStrategy<Integer> weightStrategy;
    public int count = 0;
    public int newCount = 0;

    public ValidShiftTypeConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void addShift(String id, Shift shift) {

        if (!validTypes.contains(shift.type)) {
            newCount++;
        }

    }

    @Override
    public void removeShift(String id, Shift shift) {

        if (!validTypes.contains(shift.type)) {
            newCount--;
        }

    }

    @Override
    public double evaluate() {
        newValue = weightStrategy.evaluate(newCount);
        return super.evaluate();
    }

    @Override
    public void execute() {
        super.execute();
        count = newCount;
    }

    @Override
    public void abort() {
        super.abort();
        newCount = count;
    }
}
