package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Constrains the number of working weekends.
 *
 * @author Lucas Kletzander
 */
public class WeekendCountConstraint extends ShiftArrangementConstraint {

    public Map<String, int[]> shiftCount = new HashMap<>();
    public Map<String, int[]> newShiftCount = new HashMap<>();
    public Map<String, Integer> count = new HashMap<>();
    public Map<String, Integer> newCount = new HashMap<>();
    public List<WeightStrategy<Integer>> weightStrategy;

    public WeekendCountConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        int[] current = Arrays.copyOf(shiftCount.getOrDefault(id, new int[instance.getMaxDay() / 7 + 1]), instance.getMaxDay() / 7 + 1);
        int c = count.getOrDefault(id, 0);

        for (Integer i : replacements.keySet()) {

            if (!applies(i)) {
                continue;
            }

            if (row[i] != null) {
                int weekend = instance.weekend.isWeekend(row[i], i);
                if (weekend != -1) {
                    if (current[weekend] == 1) {
                        c--;
                    }
                    current[weekend]--;
                }
            }

            if (replacements.get(i) != null) {
                int weekend = instance.weekend.isWeekend(replacements.get(i), i);
                if (weekend != -1) {
                    if (current[weekend] == 0) {
                        c++;
                    }
                    current[weekend]++;
                }
            }

        }

        newShiftCount.put(id, current);
        newCount.put(id, c);
        newValue = newValue - weightStrategy.stream().mapToDouble(i -> i.evaluate(count.getOrDefault(id, 0))).sum() + weightStrategy.stream().mapToDouble(i -> i.evaluate(newCount.get(id))).sum();

    }

    @Override
    public void execute() {
        super.execute();
        count.putAll(newCount);
        shiftCount.putAll(newShiftCount);
    }

    @Override
    public void abort() {
        super.abort();
        newCount.clear();
        newShiftCount.clear();
    }

}
