package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Constrains sequences of weekends.
 *
 * @author Lucas Kletzander
 */
public class WeekendSequenceConstraint extends ShiftArrangementConstraint {

    public Map<String, int[]> shiftCount = new HashMap<>();
    public Map<String, int[]> newShiftCount = new HashMap<>();
    public List<WeightStrategy<Integer>> weightStrategy;

    public WeekendSequenceConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        int[] current = shiftCount.getOrDefault(id, new int[instance.getMaxDay() / 7 + 1]);
        newValue -= evaluate(current);

        for (Integer i : replacements.keySet()) {

            if (!applies(i)) {
                continue;
            }

            int weekendOld = instance.weekend.isWeekend(row[i], i);
            int weekendNew = instance.weekend.isWeekend(replacements.get(i), i);

            if (weekendOld == weekendNew) {
                continue;
            }

            if (weekendOld != -1 && current[weekendOld] > 0) {
                current[weekendOld]--;
            }

            if (weekendNew != -1) {
                current[weekendNew]++;
            }

        }

        newValue += evaluate(current);
        newShiftCount.put(id, current);

    }

    public double evaluate(int[] current) {

        int val = 0;
        int curStart = -1;

        for (int i = 0; i < current.length; i++) {

            if (current[i] > 0) {

                if (curStart < 0) {
                    curStart = i;
                }

            } else {

                if (curStart >= 0) {
                    int finalI = i;
                    int finalCurStart = curStart;
                    val += weightStrategy.stream().mapToDouble(j -> j.evaluate(finalI - finalCurStart)).sum();
                    curStart = -1;
                }

            }

        }

        return val;

    }

    @Override
    public void execute() {
        super.execute();
        shiftCount.putAll(newShiftCount);
    }

    @Override
    public void abort() {
        super.abort();
        newShiftCount.clear();
    }

}
