package at.ac.tuwien.dbai.ges.solver.constraint.shift;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Constrains the workload.
 *
 * @author Lucas Kletzander
 */
public class WorkloadConstraint extends ShiftArrangementConstraint {

    public Map<String, Integer> values = new HashMap<>();
    public Map<String, Integer> newValues = new HashMap<>();
    public boolean includeBreakTime = false;//TODO currently ignored
    public List<WeightStrategy<Integer>> weightStrategy;

    public WorkloadConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void changeRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        for (Integer pos : replacements.keySet()) {

            if (!applies(pos)) {
                continue;
            }

            Integer old = newValues.getOrDefault(id, values.getOrDefault(id, 0));
            newValue -= weightStrategy.stream().mapToDouble(i -> i.evaluate(old)).sum();
            Integer val = old - getWorkload(row[pos]) + getWorkload(replacements.get(pos));
            newValue += weightStrategy.stream().mapToDouble(i -> i.evaluate(val)).sum();
            newValues.put(id, val);

        }

    }

    @Override
    public void initRow(String id, Shift[] row, Map<Integer, Shift> replacements) {

        Integer val = 0;

        for (Integer pos : replacements.keySet()) {

            if (pos < instance.history || !applies(pos)) {
                continue;
            }

            val += getWorkload(replacements.get(pos));

        }

        Integer finalVal = val;
        newValue += weightStrategy.stream().mapToDouble(i -> i.evaluate(finalVal)).sum();
        newValues.put(id, val);

    }

    // Workload belongs to the day the shift is assigned to
    private int getWorkload(Shift shift) {

        if (shift == null) {
            return 0;
        }

        return shift.end - shift.start - (includeBreakTime ? 0 : shift.breakTime);

    }

    // Workload belongs to the day where it takes place
    private int getWorkload(int day, Shift shift) {

        if (shift == null) {
            return 0;
        }

        return Math.min(day * 24 * 60 + shift.end, (endDay == -1 ? Integer.MAX_VALUE : (endDay + 1) * 24 * 60)) - Math.max(day * 24 * 60 + shift.start, startDay * 24 * 60);

    }

    @Override
    public void execute() {
        super.execute();
        values.putAll(newValues);
        newValues.clear();
    }

    @Override
    public void abort() {
        super.abort();
        newValues.clear();
    }

}
