package at.ac.tuwien.dbai.ges.solver.constraint.task;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.TaskConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.solution.TaskInstance;

import java.util.ArrayList;
import java.util.List;

/**
 * Constrains the workforce required for a task.
 *
 * @author Lucas Kletzander
 */
public class TaskRequirementConstraint extends TaskConstraint {

    public int count;
    public int newCount;
    public List<WeightStrategy<Integer>> weightStrategy = new ArrayList<>();

    public TaskRequirementConstraint(Instance instance) {
        super(instance);
    }

    @Override
    public void addTask(TaskInstance task) {
        newCount++;
    }

    @Override
    public void removeTask(TaskInstance task) {
        newCount--;
    }

    @Override
    public double evaluate() {
        newValue = weightStrategy.stream().mapToDouble(i -> i.evaluate(newCount)).sum();
        return super.evaluate();
    }

    @Override
    public void execute() {
        super.execute();
        count = newCount;
    }

    @Override
    public void abort() {
        super.abort();
        newCount = count;
    }

    @Override
    public TaskRequirementConstraint clone() {

        TaskRequirementConstraint constraint = new TaskRequirementConstraint(instance);
        constraint.weightStrategy = weightStrategy;
        return constraint;

    }

}
