package at.ac.tuwien.dbai.ges.solver.constraint.weight;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.Constraint;

import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

/**
 * Provides penalty weights for hard constraints. Each constraint class can have an individual weighting function.
 * The default is to prevent hard constraint violations (return NaN).
 *
 * @author Lucas Kletzander
 */
public class HardConstraintWeightProvider {

    public final Function<Integer, Double> defaultInt;
    public final Function<Double, Double> defaultDouble;
    public final Map<Class<? extends Constraint>, Function<Integer, Double>> intMapping = new HashMap<>();
    public final Map<Class<? extends Constraint>, Function<Double, Double>> doubleMapping = new HashMap<>();

    public HardConstraintWeightProvider() {
        defaultInt = value -> value > 0 ? Double.NaN : 0.0;
        defaultDouble = value -> value > 0.0 ? Double.NaN : 0.0;
    }

    public HardConstraintWeightProvider(Function<Integer, Double> defaultInt, Function<Double, Double> defaultDouble) {
        this.defaultInt = defaultInt;
        this.defaultDouble = defaultDouble;
    }

    public Function<Integer, Double> getIntWeightFunction(Class<? extends Constraint> clazz) {
        return intMapping.getOrDefault(clazz, defaultInt);
    }

    public Function<Double, Double> getDoubleWeightFunction(Class<? extends Constraint> clazz) {
        return doubleMapping.getOrDefault(clazz, defaultDouble);
    }

}
