package at.ac.tuwien.dbai.ges.solver.constraint.weight;

import java.util.function.Function;

/**
 * Abstract strategy for evaluation composed of a difference function calculating the value to weigh and a function to calculate the actual weight from this value.
 *
 * @author Lucas Kletzander
 */
public class WeightStrategy<T extends Number> {

    public double factor = 1.0;
    public Function<T, T> difference;
    public Function<T, Double> function;
    public Type type = Type.UNKNOWN;

    public WeightStrategy(Function<T, T> difference, Function<T, Double> function) {
        this.difference = difference;
        this.function = function;
    }

    public WeightStrategy(Function<T, T> difference, Function<T, Double> function, boolean isMin) {
        this(difference, function);
        this.type = isMin ? Type.MIN : Type.MAX;
    }

    public double evaluate(T value) {
        return factor * function.apply(difference.apply(value));
    }

    public enum Type {
        MIN, MAX, UNKNOWN
    }

}
