package at.ac.tuwien.dbai.ges.solver.converter;

import at.ac.tuwien.dbai.ges.schema.*;
import at.ac.tuwien.dbai.ges.solver.definition.*;

/**
 * Converts the full scheduling horizon.
 *
 * @author Lucas Kletzander
 */
public class HorizonConverter extends Converter<SchedulingHorizon, Instance> {

    public HorizonConverter(ConversionContext context) {
        super(SchedulingHorizon.class, Instance.class, context);
    }

    @Override
    public void convert(SchedulingHorizon source, Instance target) throws ConversionException {

        context.horizon = source;
        context.instance = target;

        target.instanceId = source.getID();
        context.provider.getConverter(General.class, Instance.class).convert(source.getGeneral(), target);

        if (source.getTasks() != null) {
            context.provider.getConverter(Tasks.class, TaskDefinition.class).convert(source.getTasks(), target.taskDefinition);
        }

        context.provider.getConverter(Shifts.class, ShiftDefinition.class).convert(source.getShifts(), target.shiftDefinition);

        if (source.getBreaks() != null) {
            context.provider.getConverter(Breaks.class, BreakDefinition.class).convert(source.getBreaks(), target.breakDefinition);
        }

        if (source.getEmployees().getSkills() != null) {
            context.provider.getConverter(Skills.class, SkillDefinition.class).convert(source.getEmployees().getSkills(), target.skillDefinition);
        }
        if (source.getEmployees().getContracts() != null) {
            context.provider.getConverter(Contracts.class, ContractDefinition.class).convert(source.getEmployees().getContracts(), target.contractDefinition);
        }
        context.provider.getConverter(Employees.class, EmployeeDefinition.class).convert(source.getEmployees(), target.employeeDefinition);
        target.employeeDefinition.updateSkillCache(target.contractDefinition);

        context.provider.getConverter(Demands.class, Instance.class).convert(source.getDemands(), target);

    }

}
