package at.ac.tuwien.dbai.ges.solver.converter;

import at.ac.tuwien.dbai.ges.schema.solution.Employee;
import at.ac.tuwien.dbai.ges.schema.solution.Schedule;
import at.ac.tuwien.dbai.ges.solver.solution.Break;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;
import at.ac.tuwien.dbai.ges.solver.solution.Solution;
import at.ac.tuwien.dbai.ges.solver.solution.Task;

import java.util.Comparator;
import java.util.TreeMap;

/**
 * Transforms the solution to the XML format.
 *
 * @author Lucas Kletzander
 */
public class ResultConverter extends Converter<Solution, Schedule> {

    private final String specification;

    public ResultConverter(ConversionContext context, String specification) {
        super(Solution.class, Schedule.class, context);
        this.specification = specification;
    }


    @Override
    public void convert(Solution source, Schedule target) throws ConversionException {

        target.setSpecification(specification);

        source.schedule.values().stream().sorted(Comparator.comparing(o -> o.id)).forEach(row -> {

            Employee employee = new Employee();
            employee.setID(row.id);

            for (int i = context.instance.history; i < row.shifts.length; i++) {

                Shift s = row.shifts[i];
                if (s == null) {
                    continue;
                }

                Employee.Shift shift = new Employee.Shift();
                shift.setType(s.type);
                shift.setDay(String.valueOf(i - context.instance.history));//TODO use date here when input uses date?
                shift.setStartTime(transformTimePoint(s.start));
                shift.setEndTime(transformTimePoint(s.end));

                TreeMap<Integer, Object> content = new TreeMap<>();

                for (Break b : s.breaks.values()) {

                    Employee.Shift.Break br = new Employee.Shift.Break();
                    br.setType(b.id);
                    br.setStartTime(transformTimePoint(b.start));
                    br.setEndTime(transformTimePoint(b.end));
                    content.put(b.start, br);

                }

                for (Task t : s.tasks.values()) {

                    Employee.Shift.Task task = new Employee.Shift.Task();
                    task.setID(t.demandId.substring(t.demandId.indexOf(":") + 1));
                    task.setStartTime(transformTimePoint(t.start));
                    task.setEndTime(transformTimePoint(t.end));
                    content.put(t.start, task);

                }

                content.keySet().forEach(j -> shift.getTaskOrBreak().add(content.get(j)));
                employee.getShift().add(shift);

            }

            target.getEmployee().add(employee);

        });

    }

    private String transformTimePoint(int timePoint) {

        String value = "";
        if (timePoint >= 24 * 60) {
            value += (timePoint / (24 * 60)) + ".";
        }
        timePoint %= (24 * 60);

        return value + String.format("%02d:%02d", timePoint / 60, timePoint % 60);

    }

}
