package at.ac.tuwien.dbai.ges.solver.converter;

import at.ac.tuwien.dbai.ges.schema.solution.Employee;
import at.ac.tuwien.dbai.ges.schema.solution.Schedule;
import at.ac.tuwien.dbai.ges.solver.constraint.solver.OverlapConstraint;
import at.ac.tuwien.dbai.ges.solver.solution.*;

/**
 * Transforms a solution from XML to the internal format.
 *
 * @author Lucas Kletzander
 */
public class SolutionConverter extends Converter<Schedule, Solution> {

    public SolutionConverter(ConversionContext context) {
        super(Schedule.class, Solution.class, context);
    }

    @Override
    public void convert(Schedule source, Solution target) throws ConversionException {

        for (Employee row : source.getEmployee()) {

            OverlapConstraint overlap = new OverlapConstraint(context.instance);
            overlap.strategy = context.weight.convertHardInt(overlap);
            EmployeeSchedule schedule = target.schedule.get(row.getID());//TODO does not work for variable employees

            for (Employee.Shift shift : row.getShift()) {

                Shift s = new Shift();
                s.type = shift.getType();
                s.start = DateTimeConverter.convertTimePoint(shift.getStartTime());
                s.end = DateTimeConverter.convertTimePoint(shift.getEndTime());
                int day = context.dateTime.convertDateOrDay(shift.getDay());

                for (Object object : shift.getTaskOrBreak()) {

                    if (object instanceof Employee.Shift.Break) {

                        Employee.Shift.Break b = (Employee.Shift.Break) object;
                        Break br = new Break();
                        br.id = (b.getType());
                        br.start = DateTimeConverter.convertTimePoint(b.getStartTime());
                        br.end = DateTimeConverter.convertTimePoint(b.getEndTime());
                        s.addBreak(br);

                    } else if (object instanceof Employee.Shift.Task) {

                        Employee.Shift.Task t = (Employee.Shift.Task) object;
                        Task task = new Task();
                        task.start = DateTimeConverter.convertTimePoint(t.getStartTime());
                        int dayDiff = task.start / (24 * 60);
                        task.demandId = (day + dayDiff) + ":" + t.getID();
                        task.start -= dayDiff * 24 * 60;
                        task.end = DateTimeConverter.convertTimePoint(t.getEndTime()) - dayDiff * 24 * 60;
                        s.tasks.put(task.start, task);

                        TaskInstance inst = new TaskInstance(task.demandId, day + dayDiff);
                        target.addTaskInstance(inst);

                        inst.employees.add(row.getID());
                        if (inst.parts.isEmpty() || inst.parts.get(inst.parts.size() - 1).start < task.start) {
                            inst.parts.add(task);
                        }

                    }

                }

                schedule.shifts[day] = s;

            }

            target.addEmployeeSchedule(schedule);

        }

    }

}
