package at.ac.tuwien.dbai.ges.solver.converter.breaks;

import at.ac.tuwien.dbai.ges.schema.BreakConfigurations;
import at.ac.tuwien.dbai.ges.schema.BreakTypes;
import at.ac.tuwien.dbai.ges.schema.Breaks;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;
import at.ac.tuwien.dbai.ges.solver.definition.BreakConfiguration;
import at.ac.tuwien.dbai.ges.solver.definition.BreakDefinition;
import at.ac.tuwien.dbai.ges.solver.definition.BreakType;

/**
 * Converts break definitions.
 *
 * @author Lucas Kletzander
 */
public class BreakConverter extends Converter<Breaks, BreakDefinition> {

    public BreakConverter(ConversionContext context) {
        super(Breaks.class, BreakDefinition.class, context);
    }

    @Override
    public void convert(Breaks source, BreakDefinition target) throws ConversionException {

        Converter<BreakTypes.BreakType, BreakType> typeConverter = context.provider.getConverter(BreakTypes.BreakType.class, BreakType.class);

        for (BreakTypes.BreakType type : source.getBreakTypes().getBreakType()) {
            BreakType t = typeConverter.createAndConvert(type);
            target.types.put(t.id, t);
        }

        Converter<BreakConfigurations.BreakConfiguration, BreakConfiguration> configConverter = context.provider.getConverter(BreakConfigurations.BreakConfiguration.class, BreakConfiguration.class);

        for (BreakConfigurations.BreakConfiguration config : source.getBreakConfigurations().getBreakConfiguration()) {
            target.configurations.add(configConverter.createAndConvert(config));
        }

    }

}
