package at.ac.tuwien.dbai.ges.solver.converter.constraint;

import at.ac.tuwien.dbai.ges.schema.GlobalShiftConstraints;
import at.ac.tuwien.dbai.ges.schema.WeightedTimeSpan;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.AverageShiftLengthConstraint;
import at.ac.tuwien.dbai.ges.solver.converter.*;

import javax.xml.bind.JAXBElement;

/**
 * Converts the average shift length constraint.
 *
 * @author Lucas Kletzander
 */
public class AverageShiftLengthConverter extends Converter<GlobalShiftConstraints.AverageShiftLength, AverageShiftLengthConstraint> {

    public AverageShiftLengthConverter(ConversionContext context) {
        super(GlobalShiftConstraints.AverageShiftLength.class, AverageShiftLengthConstraint.class, context);
    }

    @Override
    public void convert(GlobalShiftConstraints.AverageShiftLength source, AverageShiftLengthConstraint target) throws ConversionException {

        if (source.getShiftTypes() != null) {
            target.types = ConverterUtil.convertShiftTypes(context.instance.shiftDefinition, source.getShiftTypes());
        }

        for (JAXBElement<WeightedTimeSpan> border : source.getMinOrMax()) {

            switch (border.getName().toString()) {
                case "Min":
                    target.weightStrategy.add(context.weight.convertWeightedTimeSpanDouble(target, border.getValue(), true));
                    break;
                case "Max":
                    target.weightStrategy.add(context.weight.convertWeightedTimeSpanDouble(target, border.getValue(), false));
                    break;
                default:
                    throw new ConversionException("Unknown element " + border.getName());
            }

        }

    }

}
