package at.ac.tuwien.dbai.ges.solver.converter.constraint;

import at.ac.tuwien.dbai.ges.schema.SequenceCount;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.ShiftCountConstraint;
import at.ac.tuwien.dbai.ges.solver.converter.*;
import at.ac.tuwien.dbai.ges.solver.converter.shift.ShiftMatcherConverter;

/**
 * Converts count constraints.
 *
 * @author Lucas Kletzander
 */
public class CountConstraintConverter extends Converter<SequenceCount, ShiftCountConstraint> {

    public CountConstraintConverter(ConversionContext context) {
        super(SequenceCount.class, ShiftCountConstraint.class, context);
    }

    @Override
    public void convert(SequenceCount source, ShiftCountConstraint target) throws ConversionException {

        if (source.getStartDay() != null) {
            target.startDay = context.dateTime.convertDateOrDay(source.getStartDay());
        }

        if (source.getEndDay() != null) {
            target.endDay = context.dateTime.convertDateOrDay(source.getEndDay());
        }

        target.label = source.getLabel();
        target.filter = ShiftMatcherConverter.convertMatch(context, source.getNoShift(), source.getAnyShift(), source.getAnyOrNoShift(), source.getShiftTypes());
        target.weightStrategy = context.weight.convertMinMaxStrategyList(target, source.getMinOrMax());

    }

}
