package at.ac.tuwien.dbai.ges.solver.converter.constraint;

import at.ac.tuwien.dbai.ges.schema.ShiftTypes;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.EmployeeLimitConstraint;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;

/**
 * Converts employee limit constraints.
 *
 * @author Lucas Kletzander
 */
public class EmployeeLimitConverter extends Converter<ShiftTypes.ShiftType.EmployeeLimits.EmployeeLimit, EmployeeLimitConstraint> {

    public EmployeeLimitConverter(ConversionContext context) {
        super(ShiftTypes.ShiftType.EmployeeLimits.EmployeeLimit.class, EmployeeLimitConstraint.class, context);
    }

    @Override
    public void convert(ShiftTypes.ShiftType.EmployeeLimits.EmployeeLimit source, EmployeeLimitConstraint target) throws ConversionException {

        if (source.getStartDay() != null) {
            target.startDay = context.dateTime.convertDateOrDay(source.getStartDay());
        }

        if (source.getEndDay() != null) {
            target.endDay = context.dateTime.convertDateOrDay(source.getEndDay());
        }

        target.weightStrategy = context.weight.convertMinMaxStrategyList(target, source.getMinOrMax());

    }

}
