package at.ac.tuwien.dbai.ges.solver.converter.constraint;

import at.ac.tuwien.dbai.ges.schema.ForbiddenSequence;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.ForbiddenSequenceConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;
import at.ac.tuwien.dbai.ges.solver.converter.shift.ShiftMatcherConverter;

import javax.xml.bind.JAXBElement;
import java.util.ArrayList;

/**
 * Converts forbidden sequence constraints.
 *
 * @author Lucas Kletzander
 */
public class ForbiddenSequenceConstraintConverter extends Converter<ForbiddenSequence, ForbiddenSequenceConstraint> {

    public ForbiddenSequenceConstraintConverter(ConversionContext context) {
        super(ForbiddenSequence.class, ForbiddenSequenceConstraint.class, context);
    }

    @Override
    public void convert(ForbiddenSequence source, ForbiddenSequenceConstraint target) throws ConversionException {

        target.validDays = context.dateTime.convertDayOptions(source.getDay(), source.getWeekDay(), source.getDays());
        target.weightStrategy = new WeightStrategy<>(context.weight.convertMinOrMaxInt(0, false),
                source.getWeight() == null ? context.weight.hardConstraintWeight.getIntWeightFunction(target.getClass()) : context.weight.convertWeight(source.getFunction(), source.getWeight()));

        target.pattern = new ArrayList<>();
        for (JAXBElement<?> element : source.getNoShiftOrAnyShiftOrAnyOrNoShift()) {
            target.pattern.add(ShiftMatcherConverter.convertMatch(context, element));
        }

    }

}
