package at.ac.tuwien.dbai.ges.solver.converter.constraint;

import at.ac.tuwien.dbai.ges.schema.WeightedTimeSpan;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.MinWeekRestTimeConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.weight.WeightStrategy;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;
import at.ac.tuwien.dbai.ges.solver.converter.DateTimeConverter;

/**
 * Converts min weekly rest time constraints.
 *
 * @author Lucas Kletzander
 */
public class MinWeekRestTimeConverter extends Converter<WeightedTimeSpan, MinWeekRestTimeConstraint> {

    public MinWeekRestTimeConverter(ConversionContext context) {
        super(WeightedTimeSpan.class, MinWeekRestTimeConstraint.class, context);
    }

    @Override
    public void convert(WeightedTimeSpan source, MinWeekRestTimeConstraint target) throws ConversionException {

        target.minDuration = DateTimeConverter.convertTimeSpan(source.getValue());
        target.weightStrategy = new WeightStrategy<>(context.weight.convertMinOrMaxInt(7 * 24 * 60, false),
                source.getWeight() == null ? context.weight.hardConstraintWeight.getIntWeightFunction(targetClass) : context.weight.convertWeight(source.getFunction(), source.getWeight()),
                false);

    }

}
