package at.ac.tuwien.dbai.ges.solver.converter.constraint;

import at.ac.tuwien.dbai.ges.schema.PreAssignments;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.NoShiftConstraint;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;
import at.ac.tuwien.dbai.ges.solver.converter.DateTimeConverter;

/**
 * Converts no shift constraints.
 *
 * @author Lucas Kletzander
 */
public class NoShiftConverter extends Converter<PreAssignments.NoShift, NoShiftConstraint> {

    public NoShiftConverter(ConversionContext context) {
        super(PreAssignments.NoShift.class, NoShiftConstraint.class, context);
    }

    @Override
    public void convert(PreAssignments.NoShift source, NoShiftConstraint target) throws ConversionException {

        target.strategy = context.weight.convertHardInt(target);

        if (source.getStartDay() != null) {
            target.startDay = context.dateTime.convertDateOrDay(source.getStartDay());
        }
        if (source.getEndDay() != null) {
            target.endDay = context.dateTime.convertDateOrDay(source.getEndDay());
        }
        if (source.getStartTime() != null) {
            target.startTime = DateTimeConverter.convertTimePoint(source.getStartTime());
        }
        if (source.getEndTime() != null) {
            target.endTime = DateTimeConverter.convertTimePoint(source.getEndTime());
        }

    }

}
