package at.ac.tuwien.dbai.ges.solver.converter.constraint;

import at.ac.tuwien.dbai.ges.schema.SequenceCount;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.ShiftSequenceConstraint;
import at.ac.tuwien.dbai.ges.solver.converter.*;
import at.ac.tuwien.dbai.ges.solver.converter.shift.ShiftMatcherConverter;

/**
 * Converts sequence constraints.
 *
 * @author Lucas Kletzander
 */
public class SequenceConstraintConverter extends Converter<SequenceCount, ShiftSequenceConstraint> {

    public SequenceConstraintConverter(ConversionContext context) {
        super(SequenceCount.class, ShiftSequenceConstraint.class, context);
    }

    @Override
    public void convert(SequenceCount source, ShiftSequenceConstraint target) throws ConversionException {

        if (source.getStartDay() != null) {
            target.startDay = context.dateTime.convertDateOrDay(source.getStartDay());
        }

        if (source.getEndDay() != null) {
            target.endDay = context.dateTime.convertDateOrDay(source.getEndDay());
        }

        target.label = source.getLabel();
        target.filter = ShiftMatcherConverter.convertMatch(context, source.getNoShift(), source.getAnyShift(), source.getAnyOrNoShift(), source.getShiftTypes());
        target.weightStrategy = context.weight.convertMinMaxStrategyList(target, source.getMinOrMax());

    }

}
