package at.ac.tuwien.dbai.ges.solver.converter.constraint;

import at.ac.tuwien.dbai.ges.schema.EmployeePair;
import at.ac.tuwien.dbai.ges.schema.PairingType;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.ShiftPairConstraint;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;
import at.ac.tuwien.dbai.ges.solver.converter.shift.ShiftMatcherConverter;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.function.Predicate;

/**
 * Converts shift pair constraints.
 *
 * @author Lucas Kletzander
 */
public class ShiftPairConverter extends Converter<EmployeePair, ShiftPairConstraint> {

    public final boolean pair;

    public ShiftPairConverter(ConversionContext context, boolean pair) {
        super(EmployeePair.class, ShiftPairConstraint.class, context);
        this.pair = pair;
    }

    @Override
    public void convert(EmployeePair source, ShiftPairConstraint target) throws ConversionException {

        target.label = source.getLabel();
        target.weight = source.getWeight() == null ? Double.NaN : source.getWeight();
        target.validDays = context.dateTime.convertDayOptions(source.getDay(), source.getWeekDay(), source.getDays());

        target.filter = new HashMap<>();
        for (PairingType.Assignment assignment : source.getPairing().getAssignment()) {

            List<Predicate<Shift>> sequence = new ArrayList<>();
            for (Object element : assignment.getNoShiftOrAnyShiftOrAnyOrNoShift()) {
                sequence.add(ShiftMatcherConverter.convertMatch(context, element));
            }

            target.filter.computeIfAbsent(assignment.getEmployee(), i -> new ArrayList<>()).add(sequence);

        }

        target.pair = pair;

    }

}
