package at.ac.tuwien.dbai.ges.solver.converter.constraint;

import at.ac.tuwien.dbai.ges.schema.EmployeeList;
import at.ac.tuwien.dbai.ges.schema.WeightedDouble;
import at.ac.tuwien.dbai.ges.schema.WeightedInteger;
import at.ac.tuwien.dbai.ges.solver.constraint.employee.VariableEmployeeConstraint;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;

import javax.xml.bind.JAXBElement;
import java.util.ArrayList;

/**
 * Converts variable employee constraints.
 *
 * @author Lucas Kletzander
 */
public class VariableEmployeeConstraintConverter extends Converter<EmployeeList.VariableEmployee, VariableEmployeeConstraint> {

    public VariableEmployeeConstraintConverter(ConversionContext context) {
        super(EmployeeList.VariableEmployee.class, VariableEmployeeConstraint.class, context);
    }

    @Override
    public void convert(EmployeeList.VariableEmployee source, VariableEmployeeConstraint target) throws ConversionException {

        target.match = source.getID();
        target.countWeights = new ArrayList<>();
        target.fractionWeights = new ArrayList<>();

        for (JAXBElement<?> element : source.getMinNumberOrMaxNumberOrMinFraction()) {

            switch (element.getName().getLocalPart()) {
                case "MinNumber":
                    target.countWeights.add(context.weight.convertWeightedInteger(target, (WeightedInteger) element.getValue(), true));
                    break;
                case "MaxNumber":
                    target.countWeights.add(context.weight.convertWeightedInteger(target, (WeightedInteger) element.getValue(), false));
                    break;
                case "MinFraction":
                    target.fractionWeights.add(context.weight.convertWeightedDouble(target, (WeightedDouble) element.getValue(), true));
                    break;
                case "MaxFraction":
                    target.fractionWeights.add(context.weight.convertWeightedDouble(target, (WeightedDouble) element.getValue(), false));
                    break;
                default:
                    throw new ConversionException("Unknown element " + element.getName().getLocalPart());
            }

        }

    }

}
