package at.ac.tuwien.dbai.ges.solver.converter.constraint;

import at.ac.tuwien.dbai.ges.schema.WeekendSequenceCount;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.WeekendCountConstraint;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;

/**
 * Converts weekend count constraints.
 *
 * @author Lucas Kletzander
 */
public class WeekendCountConstraintConverter extends Converter<WeekendSequenceCount, WeekendCountConstraint> {

    public WeekendCountConstraintConverter(ConversionContext context) {
        super(WeekendSequenceCount.class, WeekendCountConstraint.class, context);
    }

    @Override
    public void convert(WeekendSequenceCount source, WeekendCountConstraint target) throws ConversionException {

        if (source.getStartDay() != null) {
            target.startDay = context.dateTime.convertDateOrDay(source.getStartDay());
        }

        if (source.getEndDay() != null) {
            target.endDay = context.dateTime.convertDateOrDay(source.getEndDay());
        }

        target.weightStrategy = context.weight.convertMinMaxStrategyList(target, source.getMinOrMax());

    }

}
