package at.ac.tuwien.dbai.ges.solver.converter.constraint;

import at.ac.tuwien.dbai.ges.schema.WorkloadType;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.WorkloadConstraint;
import at.ac.tuwien.dbai.ges.solver.converter.*;

import java.util.ArrayList;

/**
 * Converts workload constraints.
 *
 * @author Lucas Kletzander
 */
public class WorkloadConstraintConverter extends Converter<WorkloadType, WorkloadConstraint> {

    public WorkloadConstraintConverter(ConversionContext context) {
        super(WorkloadType.class, WorkloadConstraint.class, context);
    }

    @Override
    public void convert(WorkloadType source, WorkloadConstraint target) throws ConversionException {

        if (source.getStartDay() != null) {
            target.startDay = context.dateTime.convertDateOrDay(source.getStartDay());
        }

        if (source.getEndDay() != null) {
            target.endDay = context.dateTime.convertDateOrDay(source.getEndDay());
        }

        target.weightStrategy = new ArrayList<>();
        if (source.getMin() != null) {
            target.weightStrategy.add(context.weight.convertWeightedTimeSpanInt(target, source.getMin(), true));
        }
        if (source.getMax() != null) {
            target.weightStrategy.add(context.weight.convertWeightedTimeSpanInt(target, source.getMax(), false));
        }

        target.includeBreakTime = ConverterUtil.convertWorkUnit(source.getUnit());

    }

}
