package at.ac.tuwien.dbai.ges.solver.converter.demand;

import at.ac.tuwien.dbai.ges.schema.Demands;
import at.ac.tuwien.dbai.ges.schema.ShiftCover;
import at.ac.tuwien.dbai.ges.schema.TaskCover;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;
import at.ac.tuwien.dbai.ges.solver.definition.Instance;
import at.ac.tuwien.dbai.ges.solver.demand.ShiftDemands;
import at.ac.tuwien.dbai.ges.solver.demand.TaskDemands;

/**
 * Converts various demand types.
 *
 * @author Lucas Kletzander
 */
public class DemandsConverter extends Converter<Demands, Instance> {

    public DemandsConverter(ConversionContext context) {
        super(Demands.class, Instance.class, context);
    }

    @Override
    public void convert(Demands source, Instance target) throws ConversionException {

        if (!source.getShiftCover().isEmpty()) {

            target.demandType = Instance.DemandType.SHIFT;
            target.shiftDemands = new ShiftDemands(context.instance.periodLength);

            Converter<ShiftCover, ShiftDemands> converter = context.provider.getConverter(ShiftCover.class, ShiftDemands.class);

            for (ShiftCover cover : source.getShiftCover()) {
                converter.convert(cover, target.shiftDemands);
            }

        } else if (!source.getTimeCover().isEmpty()) {
            //TODO not yet implemented
        } else if (!source.getTaskCover().isEmpty()) {

            target.demandType = Instance.DemandType.TASK;
            target.taskDemands = new TaskDemands(context.instance.history + context.instance.periodLength);

            Converter<TaskCover, TaskDemands> converter = context.provider.getConverter(TaskCover.class, TaskDemands.class);

            for (TaskCover cover : source.getTaskCover()) {
                converter.convert(cover, target.taskDemands);
            }

        } else {
            throw new ConversionException("No demand specified");
        }

    }

}
