package at.ac.tuwien.dbai.ges.solver.converter.demand;

import at.ac.tuwien.dbai.ges.schema.ShiftCover;
import at.ac.tuwien.dbai.ges.schema.WeightedIntSkill;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.ShiftRequirementConstraint;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;
import at.ac.tuwien.dbai.ges.solver.demand.ShiftDemand;
import at.ac.tuwien.dbai.ges.solver.demand.ShiftDemands;

import javax.xml.bind.JAXBElement;
import java.util.stream.IntStream;

/**
 * Converts one shift cover and adds the copies of the shift request to all specified days in the demand structure.
 *
 * @author Lucas Kletzander
 */
public class ShiftDemandConverter extends Converter<ShiftCover, ShiftDemands> {

    public ShiftDemandConverter(ConversionContext context) {
        super(ShiftCover.class, ShiftDemands.class, context);
    }

    @Override
    public void convert(ShiftCover source, ShiftDemands target) throws ConversionException {

        ShiftDemand demand = new ShiftDemand();
        demand.id = source.getID();
        demand.shift = source.getShift();

        ShiftRequirementConstraint requirement = new ShiftRequirementConstraint(context.instance);

        for (JAXBElement<WeightedIntSkill> req : source.getRequirements().getMinOrMax()) {

            ShiftRequirementConstraint constraint = req.getValue().getSkill() == null ? requirement : demand.skillConstraints.computeIfAbsent(req.getValue().getSkill(), i -> new ShiftRequirementConstraint(context.instance));
            switch (req.getName().getLocalPart().toLowerCase()) {
                case "min":
                    constraint.weightStrategy.add(context.weight.convertWeightedInteger(constraint, req.getValue(), true));
                    break;
                case "max":
                    constraint.weightStrategy.add(context.weight.convertWeightedInteger(constraint, req.getValue(), false));
                    break;
                default:
                    throw new ConversionException("Unknown constraint " + req.getName());
            }

        }

        if (!requirement.weightStrategy.isEmpty()) {
            demand.requirement = requirement;
        }

        boolean[] days = context.dateTime.convertDayOptions(source.getDay(), source.getWeekDay(), source.getDays());
        IntStream.range(0, days.length).filter(i -> days[i]).forEach(i -> {
            target.demand.get(i).put(demand.shift, demand.clone());
        });

    }

}
