package at.ac.tuwien.dbai.ges.solver.converter.employee;

import at.ac.tuwien.dbai.ges.schema.Contracts;
import at.ac.tuwien.dbai.ges.schema.WeightedTimeSpan;
import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.Constraint;
import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.ValidShiftTypeConstraint;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;
import at.ac.tuwien.dbai.ges.solver.definition.Contract;
import at.ac.tuwien.dbai.ges.solver.definition.ContractDefinition;

import javax.xml.bind.JAXBElement;

/**
 * Converts contracts.
 *
 * @author Lucas Kletzander
 */
public class ContractConverter extends Converter<Contracts, ContractDefinition> {

    public ContractConverter(ConversionContext context) {
        super(Contracts.class, ContractDefinition.class, context);
    }

    @Override
    public void convert(Contracts source, ContractDefinition target) throws ConversionException {

        for (Contracts.Contract contract : source.getContract()) {

            Contract c = new Contract();
            c.id = contract.getID();
            c.parent = contract.getExtends();
            c.label = contract.getLabel();

            if (contract.isAllowWorkSwitchesPerShift() != null) {
                c.workSwitchesPerShift = contract.isAllowWorkSwitchesPerShift();
            }

            if (contract.getWorkSwitchLimitPerShift() != null) {
                //TODO
            }

            if (contract.getPatternConstraints() != null) {
                for (JAXBElement<?> element : contract.getPatternConstraints().getCountConstraintOrSequenceConstraintOrWeekendCount()) {
                    c.arrangementConstraints.add(context.provider.getConverter(element, ShiftArrangementConstraint.class).createAndConvert(element.getValue()));//TODO MatchConstraint
                }
            }

            if (contract.getConditionals() != null) {
                //TODO
            }

            if (contract.getProvidedSkills() != null) {
                c.skills = context.instance.skillDefinition.getSkillMap(contract.getProvidedSkills().getSkill());
            }

            c.shiftTypes = contract.getValidShiftTypes();
            if (c.shiftTypes != null) {
                ValidShiftTypeConstraint typeConstraint = new ValidShiftTypeConstraint(context.instance);
                typeConstraint.validTypes.addAll(context.instance.shiftDefinition.shiftGroups.get(c.shiftTypes));
                typeConstraint.weightStrategy = context.weight.convertHardInt(typeConstraint);
                c.shiftConstraints.add(typeConstraint);
            }

            if (contract.getFairnessConstraints() != null) {

                for (WeightedTimeSpan ts : contract.getFairnessConstraints().getLargestWorkloadGap()) {

                    Converter<WeightedTimeSpan, ? extends ShiftConstraint> converter = context.provider.getConverter(WeightedTimeSpan.class, ShiftConstraint.class);
                    c.shiftConstraints.add(converter.createAndConvert(ts));

                }

            }

            if (contract.getUsageOptional() != null) {
                //TODO
            }

            for (JAXBElement<?> element : contract.getMinRestTimeOrMinWeekRestTimeOrWorkload()) {
                Constraint constraint = context.provider.getConverter(element, Constraint.class).createAndConvert(element.getValue());
                if (constraint instanceof ShiftConstraint) {
                    c.shiftConstraints.add((ShiftConstraint) constraint);
                } else if (constraint instanceof ShiftArrangementConstraint) {
                    c.arrangementConstraints.add((ShiftArrangementConstraint) constraint);
                } else {
                    throw new ConversionException("Error converting constraint " + element.getName() + ": invalid converted type " + constraint.getClass());
                }//TODO TaskLengths
            }

            target.contracts.put(c.id, c);

        }

    }

}
