package at.ac.tuwien.dbai.ges.solver.converter.shift;

import at.ac.tuwien.dbai.ges.schema.AnyOrNoShift;
import at.ac.tuwien.dbai.ges.schema.AnyShift;
import at.ac.tuwien.dbai.ges.schema.NoShift;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import javax.xml.bind.JAXBElement;
import java.util.Objects;
import java.util.function.Predicate;

/**
 * Converts various ways to match specific sets of shift types to predicates.
 *
 * @author Lucas Kletzander
 */
public class ShiftMatcherConverter {

    public static Predicate<Shift> convertMatch(ConversionContext context, NoShift noShift, AnyShift anyShift, AnyOrNoShift anyOrNoShift, String shiftTypes) {

        if (noShift != null) {
            return Objects::isNull;
        }

        if (anyShift != null) {
            return Objects::nonNull;
        }

        if (shiftTypes != null) {
            return i -> i != null && context.instance.shiftDefinition.shiftGroups.get(shiftTypes).contains(i.type);
        }

        return i -> true;

    }

    public static Predicate<Shift> convertMatch(ConversionContext context, Object element) {

        if (element instanceof NoShift) {
            return Objects::isNull;
        }

        if (element instanceof AnyShift) {
            return Objects::nonNull;
        }

        if (element instanceof String) {
            return i -> i != null && context.instance.shiftDefinition.shiftGroups.get(element).contains(i.type);
        }

        return i -> true;

    }

    public static Predicate<Shift> convertMatch(ConversionContext context, JAXBElement<?> element) throws ConversionException {

        switch (element.getName().getLocalPart().toLowerCase()) {
            case "noshift":
                return Objects::isNull;
            case "anyshift":
                return Objects::nonNull;
            case "anyornoshift":
                return i -> true;
            case "shifttypes":
                return i -> i != null && context.instance.shiftDefinition.shiftGroups.get(element.getValue().toString()).contains(i.type);
            case "notshifttypes":
                return i -> i == null || !context.instance.shiftDefinition.shiftGroups.get(element.getValue().toString()).contains(i.type);
            default:
                throw new ConversionException("Unknown shift option " + element.getName());
        }

    }

}
