package at.ac.tuwien.dbai.ges.solver.converter.task;

import at.ac.tuwien.dbai.ges.schema.TaskGroups;
import at.ac.tuwien.dbai.ges.schema.TaskTypes;
import at.ac.tuwien.dbai.ges.schema.Tasks;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionContext;
import at.ac.tuwien.dbai.ges.solver.converter.ConversionException;
import at.ac.tuwien.dbai.ges.solver.converter.Converter;
import at.ac.tuwien.dbai.ges.solver.converter.DateTimeConverter;
import at.ac.tuwien.dbai.ges.solver.definition.TaskDefinition;
import at.ac.tuwien.dbai.ges.solver.definition.TaskType;

import java.util.ArrayList;
import java.util.List;

/**
 * Converts task types and task groups.
 *
 * @author Lucas Kletzander
 */
public class TaskConverter extends Converter<Tasks, TaskDefinition> {

    public TaskConverter(ConversionContext context) {
        super(Tasks.class, TaskDefinition.class, context);
    }

    @Override
    public void convert(Tasks source, TaskDefinition target) throws ConversionException {

        for (TaskTypes.TaskType type : source.getTaskTypes().getTaskType()) {

            TaskType t = new TaskType();
            t.id = type.getID();
            t.name = type.getName();

            if (type.getPrerequisites() != null) {
                t.prerequisites.addAll(type.getPrerequisites().getTask());
            }
            if (type.isIsWork() != null) {
                t.isWork = type.isIsWork();
            }
            if (type.getReAcquaintancePeriod() != null) {
                t.reAq = DateTimeConverter.convertTimeSpan(type.getReAcquaintancePeriod());
            }

            target.taskTypes.put(t.id, t);
            List<String> singleGroup = new ArrayList<>();
            singleGroup.add(t.id);
            target.taskGroups.put(t.id, singleGroup);

        }

        if (source.getTaskGroups() != null) {

            for (TaskGroups.TaskGroup g : source.getTaskGroups().getTaskGroup()) {

                List<String> group = new ArrayList<>(g.getTask());
                target.taskGroups.put(g.getID(), group);

            }

        }

    }

}
