package at.ac.tuwien.dbai.ges.solver.definition;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.BreakArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * A break configuration.
 *
 * @author Lucas Kletzander
 */
public class BreakConfiguration {

    public ShiftFilter filter;
    public List<BreakArrangementConstraint> constraints = new ArrayList<>();
    public boolean breakSetOrdered = false;
    public List<BreakOccurrence> orderedBreaks;
    public Map<String, BreakOccurrence> unorderedBreaks;

    public static class ShiftFilter {

        public Instance instance;
        public boolean[] validDays;
        public int minShiftLength = 0;
        public int maxShiftLength = Integer.MAX_VALUE;
        public int minShiftStart = 0;
        public int maxShiftStart = Integer.MAX_VALUE;
        public int minShiftEnd = 0;
        public int maxShiftEnd = Integer.MAX_VALUE;
        public Set<String> contracts;
        public Set<String> types;
        public int minWorkLength = 0;
        public int maxWorkLength = Integer.MAX_VALUE;

        public boolean applies(String employee, int day, Shift shift) {

            return validDays[day] &&
                    minShiftLength <= shift.getDuration() &&
                    maxShiftLength >= shift.getDuration() &&
                    minShiftStart <= shift.start &&
                    maxShiftStart >= shift.start &&
                    minShiftEnd <= shift.end &&
                    maxShiftEnd >= shift.end &&
                    (contracts == null || instance.employeeDefinition.getEmployee(employee).contracts.stream().anyMatch(i -> contracts.contains(i))) &&
                    (types == null || types.contains(shift.type)) &&
                    minWorkLength <= shift.getWorkTime() &&
                    maxWorkLength >= shift.getWorkTime();

        }

    }

    public static class BreakOccurrence {

        public String id;
        public int min = 0;
        public int max = Integer.MAX_VALUE;

    }

}
