package at.ac.tuwien.dbai.ges.solver.definition;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.BreakArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.BreakConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.Constraint;
import at.ac.tuwien.dbai.ges.solver.solution.Break;
import at.ac.tuwien.dbai.ges.solver.solution.Shift;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

/**
 * A break definition.
 *
 * @author Lucas Kletzander
 */
public class BreakDefinition {

    public List<BreakConfiguration> configurations = new ArrayList<>();
    public Map<String, BreakType> types = new HashMap<>();

    private BreakConfiguration dummy = new BreakConfiguration();

    public Stream<BreakConstraint> getBreakConstraints(Break br) {
        return types.get(br.id).localConstraints.stream();
    }

    public Stream<BreakArrangementConstraint> getBreakArrangementConstraints(String employee, int day, Shift shift) {

        return Stream.concat(
                configurations.stream()
                        .filter(i -> i.filter == null || i.filter.applies(employee, day, shift))
                        .findFirst()
                        .orElse(dummy)
                        .constraints.stream(),
                types.values().stream()
                        .flatMap(i -> i.constraints.stream()));

    }

    public Stream<Constraint> getConstraints() {

        return Stream.concat(
                types.values().stream()
                        .flatMap(i -> Stream.concat(i.localConstraints.stream(), i.constraints.stream())),
                configurations.stream()
                        .flatMap(i -> i.constraints.stream()));

    }

}
