package at.ac.tuwien.dbai.ges.solver.definition;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.Constraint;
import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.EmployeeConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftArrangementConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.shift.ShiftPairConstraint;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

/**
 * Contains all employees and provides access to their constraints.
 *
 * @author Lucas Kletzander
 */
public class EmployeeDefinition {

    public Map<String, Employee> employees = new HashMap<>();
    public Map<String, VariableEmployee> varEmployees = new HashMap<>();
    public List<ShiftPairConstraint> pairConstraints = new ArrayList<>();
    public List<EmployeeConstraint> employeeConstraints = new ArrayList<>();

    public AbstractEmployee getEmployee(String id) {

        Employee employee = employees.get(id);
        return employee == null ? varEmployees.get(id) : employee;

    }

    public void updateSkillCache(ContractDefinition contractDefinition) {

        employees.values().forEach(i -> i.updateSkillCache(contractDefinition));
        varEmployees.values().forEach(i -> i.updateSkillCache(contractDefinition));

    }

    public Stream<EmployeeConstraint> getEmployeeConstraints() {
        return employeeConstraints.stream();
    }

    public Stream<ShiftArrangementConstraint> getShiftArrangementConstraints(String employee) {
        return Stream.concat(getEmployee(employee).preferences.stream(), pairConstraints.stream());
    }

    public Stream<Constraint> getConstraints() {

        return Stream.concat(
                Stream.concat(
                        Stream.concat(
                                employees.values().stream().flatMap(i -> i.preferences.stream()),
                                varEmployees.values().stream().flatMap(i -> i.preferences.stream())),
                        pairConstraints.stream()),
                employeeConstraints.stream());

    }

}
