package at.ac.tuwien.dbai.ges.solver.definition;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.Constraint;
import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.ShiftConstraint;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

/**
 * Contains all shift definitions and access to the corresponding constraints.
 *
 * @author Lucas Kletzander
 */
public class ShiftDefinition {

    public Map<String, ShiftType> shiftTypes;
    public Map<String, List<String>> shiftGroups;
    public List<ShiftConstraint> globalShiftConstraints;
    public boolean allowBorderShifts = false;

    public ShiftDefinition() {

        shiftTypes = new HashMap<>();
        shiftGroups = new HashMap<>();
        globalShiftConstraints = new ArrayList<>();

    }

    public Stream<ShiftConstraint> getShiftConstraints(String type) {
        return Stream.concat(shiftTypes.get(type).shiftTypeConstraints.stream(), globalShiftConstraints.stream());
    }

    public Stream<Constraint> getConstraints() {
        return Stream.concat(shiftTypes.values().stream().flatMap(i -> i.shiftTypeConstraints.stream()), globalShiftConstraints.stream());
    }

}
