package at.ac.tuwien.dbai.ges.solver.definition;

import java.util.*;

/**
 * Contains definitions of skills.
 *
 * @author Lucas Kletzander
 */
public class SkillDefinition {

    public Map<String, List<Included>> skills = new HashMap<>();

    public Map<String, Double> getSkillMap(Collection<String> base) {

        Map<String, Double> skillMap = new HashMap<>();
        base.forEach(i -> skillMap.put(i, 0.0));

        Queue<String> queue = new LinkedList<>(base);

        while (!queue.isEmpty()) {

            String current = queue.poll();
            for (Included inc : skills.getOrDefault(current, new ArrayList<>())) {

                double penalty = skillMap.get(current) + inc.penalty;
                if (!skillMap.containsKey(inc.skill) || penalty < skillMap.get(inc.skill)) {
                    skillMap.put(inc.skill, penalty);
                    queue.offer(inc.skill);
                }

            }

        }

        return skillMap;

    }

    public static class Included {

        public final String skill;
        public final double penalty;

        public Included(String skill, Double penalty) {
            this.skill = skill;
            this.penalty = penalty == null ? 0.0 : penalty;
        }

    }

}
