package at.ac.tuwien.dbai.ges.solver.demand;

import at.ac.tuwien.dbai.ges.solver.constraint.hierarchy.TaskConstraint;
import at.ac.tuwien.dbai.ges.solver.constraint.task.TaskRequirementConstraint;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

/**
 * A task demand.
 *
 * @author Lucas Kletzander
 */
public class TaskDemand implements Cloneable {

    public String id;
    public String task;
    public int length;
    public List<TaskConstraint> constraints = new ArrayList<>();
    public Map<String, TaskRequirementConstraint> skillConstraints = new HashMap<>();
    public AllowSplit allowSplit = AllowSplit.NO;
    public int minSplitLength;
    public List<String> prerequisites = new ArrayList<>();

    public int earliestStart = -1;
    public int latestEnd = -1;

    public double getConstraintValue() {
        return Stream.concat(constraints.stream(), skillConstraints.values().stream()).mapToDouble(i -> i.value).sum();
    }

    @Override
    public TaskDemand clone() {

        TaskDemand demand = new TaskDemand();
        demand.id = id;
        demand.task = task;
        demand.length = length;

        for (TaskConstraint constraint : constraints) {
            demand.constraints.add(constraint.clone());
        }

        for (Map.Entry<String, TaskRequirementConstraint> entry : skillConstraints.entrySet()) {
            demand.skillConstraints.put(entry.getKey(), entry.getValue().clone());
        }

        demand.allowSplit = allowSplit;
        demand.minSplitLength = minSplitLength;
        demand.prerequisites = prerequisites;
        demand.earliestStart = earliestStart;
        demand.latestEnd = latestEnd;

        return demand;

    }

    public enum AllowSplit {
        NO, BREAKS_ONLY, YES
    }

}
