package at.ac.tuwien.dbai.ges.solver.solution;

import java.util.NavigableMap;
import java.util.TreeMap;

/**
 * A shift
 *
 * @author Lucas Kletzander
 */
public class Shift implements Cloneable {

    private static long nextId = 0;

    public long id;
    public String type;
    public int start;
    public boolean startFixed = false;
    public int end = -1;
    public boolean endFixed = false;
    public int breakTime;
    public NavigableMap<Integer, Break> breaks = new TreeMap<>();
    public NavigableMap<Integer, Task> tasks = new TreeMap<>();

    public Shift() {
        id = nextId++;
    }

    public int getDuration() {
        return end - start;
    }

    public int getWorkTime() {
        return getDuration() - breakTime;
    }

    public void addBreak(Break br) {
        breakTime += br.getDuration();
        breaks.put(br.start, br);
    }

    public void removeBreak(Break br) {
        breakTime -= br.getDuration();
        breaks.remove(br.start);
    }

    @Override
    public Shift clone() {

        Shift shift = new Shift();
        shift.type = type;
        shift.start = start;
        shift.startFixed = startFixed;
        shift.end = end;
        shift.endFixed = endFixed;
        shift.breakTime = breakTime;
        shift.breaks.putAll(breaks);
        shift.tasks.putAll(tasks);
        return shift;

    }

}
